@@ -21,4 +21,12 @@
 #
 
 # Declaring this as a namespace package
-__path__ = __import__('pkgutil').extend_path(__path__, __name__)  # @ReservedAssignment
\ No newline at end of file
+__path__ = __import__("pkgutil").extend_path(__path__, __name__)  # @ReservedAssignment
+# set the version
+try:
+    from dlg.common import version
+
+    __version__ = version.full_version
+except:
+    # This can happen when running from source
+    __version__ = "unknown"
