@@ -38,15 +38,15 @@ import tempfile
 import threading
 import time
 import types
-import collections
+import json
 
 from .. import droputils, utils
+from dlg.named_port_utils import replace_named_ports
 from ..ddap_protocol import AppDROPStates, DROPStates
-from ..drop import BarrierAppDROP, AppDROP
+from ..apps.app_base import BarrierAppDROP, AppDROP
 from ..exceptions import InvalidDropException
 from ..meta import (
     dlg_string_param,
-    dlg_dict_param,
     dlg_component,
     dlg_batch_input,
     dlg_batch_output,
@@ -139,7 +139,9 @@ def prepare_input_channel(data):
         chan = lambda: None  # a simple object where attributes can be set!
         chan.read = pipe.read
         chan.fileno = pipe.fileno
-        chan.close = types.MethodType(lambda s: close_and_remove(pipe, pipe_name), chan)
+        chan.close = types.MethodType(
+            lambda s: close_and_remove(pipe, pipe_name), chan
+        )
         return chan
 
     elif data.startswith(b"tcp://"):
@@ -168,20 +170,23 @@ class BashShellBase(object):
         super(BashShellBase, self).initialize(**kwargs)
 
         self.proc = None
-        self._inputRedirect = self._getArg(kwargs, "input_redirection", "")
-        self._outputRedirect = self._getArg(kwargs, "output_redirection", "")
-        self._cmdLineArgs = self._getArg(kwargs, "command_line_arguments", "")
-        self._applicationArgs = self._getArg(kwargs, "applicationArgs", {})
-        self._argumentPrefix = self._getArg(kwargs, "argumentPrefix", "--")
-        self._paramValueSeparator = self._getArg(kwargs, "paramValueSeparator", " ")
+        self._inputRedirect = self._popArg(kwargs, "input_redirection", "")
+        self._outputRedirect = self._popArg(kwargs, "output_redirection", "")
+        self._cmdLineArgs = self._popArg(kwargs, "command_line_arguments", "")
+        self._applicationArgs = self._popArg(kwargs, "applicationArgs", {})
+        self._argumentPrefix = self._popArg(kwargs, "argumentPrefix", "--")
+        self._paramValueSeparator = self._popArg(
+            kwargs, "paramValueSeparator", " "
+        )
 
         if not self.command:
-            self.command = self._getArg(kwargs, "command", None)
+            self.command = self._popArg(kwargs, "command", None)
             if not self.command:
                 raise InvalidDropException(
                     self, "No command specified, cannot create BashShellApp"
                 )
 
+        self.appArgs = self._applicationArgs
         self._recompute_data = {}
 
     def _run_bash(self, inputs, outputs, stdin=None, stdout=subprocess.PIPE):
@@ -197,49 +202,44 @@ class BashShellBase(object):
         output of the process is piped to. If not given it is consumed by this
         method and potentially logged.
         """
-        # we currently only support passing a path for bash apps
-        inputs_dict = collections.OrderedDict()
-        for uid, drop in inputs.items():
-            inputs_dict[uid] = drop.path
+        # logger.debug("Parameters found: %s", json.dumps(self.parameters))
+        # we only support passing a path for bash apps
+        fsInputs = {
+            uid: i for uid, i in inputs.items() if droputils.has_path(i)
+        }
+        fsOutputs = {
+            uid: o for uid, o in outputs.items() if droputils.has_path(o)
+        }
+        dataURLInputs = {
+            uid: i for uid, i in inputs.items() if not droputils.has_path(i)
+        }
+        dataURLOutputs = {
+            uid: o for uid, o in outputs.items() if not droputils.has_path(o)
+        }
 
-        outputs_dict = collections.OrderedDict()
-        for uid, drop in outputs.items():
-            outputs_dict[uid] = drop.path
-
-        session_id = (
-            self._dlg_session.sessionId if self._dlg_session is not None else ""
+        # deal with named ports
+        inport_names = (
+            self.parameters["inputs"] if "inputs" in self.parameters else []
         )
-        logger.debug(f"Parameters found: {self.parameters}")
-        # pargs, keyargs = droputils.serialize_applicationArgs(
-        #     self._applicationArgs, self._argumentPrefix, self._paramValueSeparator
-        # )
-        if "applicationArgs" in self.parameters:
-            appArgs = self.parameters["applicationArgs"]
-        else:
-            appArgs ={}
-        pargs = [arg for arg in appArgs if appArgs[arg]["positional"]]
-        pargsDict = collections.OrderedDict(zip(pargs,[None]*len(pargs)))
-        keyargs = {arg:appArgs[arg]["value"] for arg in appArgs if not appArgs[arg]["positional"]}
-        logger.debug("pargs: %s; keyargs: %s, appArgs: %s",pargs, keyargs, appArgs)
-        if "inputs" in self.parameters and isinstance(self.parameters['inputs'][0], dict):
-            keyargs = droputils.identify_named_ports(
-                            inputs_dict,
-                            self.parameters["inputs"],
-                            pargs,
-                            pargsDict,
-                            appArgs,
-                            check_len=len(inputs),
-                            mode="inputs")
-        else:
-            for i in range(min(len(inputs), len(pargs))):
-                keyargs.update({pargs[i]: list(inputs.values())[i]})
-        keyargs = droputils.serialize_kwargs(keyargs, 
-            prefix=self._argumentPrefix,
-            separator=self._paramValueSeparator)
-        pargs = list(pargsDict.values())
-        argumentString = f"{' '.join(pargs + keyargs)}"  # add kwargs to end of pargs
+        outport_names = (
+            self.parameters["outputs"] if "outputs" in self.parameters else []
+        )
+        keyargs, pargs = replace_named_ports(
+            inputs.items(),
+            outputs.items(),
+            inport_names,
+            outport_names,
+            self.appArgs,
+            argumentPrefix=self._argumentPrefix,
+            separator=self._paramValueSeparator,
+        )
+        argumentString = f"{' '.join(map(str,pargs + keyargs))}"  # add kwargs to end of pargs
         # complete command including all additional parameters and optional redirects
-        cmd = f"{self.command} {argumentString} {self._cmdLineArgs} "
+        if len(argumentString.strip()) > 0:
+            # the _cmdLineArgs would very likely make the command line invalid
+            cmd = f"{self.command} {argumentString} "
+        else:
+            cmd = f"{self.command} {argumentString} {self._cmdLineArgs} "
         if self._outputRedirect:
             cmd = f"{cmd} > {self._outputRedirect}"
         if self._inputRedirect:
@@ -247,20 +247,13 @@ class BashShellBase(object):
         cmd = cmd.strip()
 
         app_uid = self.uid
-        # self.run_bash(self._command, self.uid, session_id, *args, **kwargs)
 
         # Replace inputs/outputs in command line with paths or data URLs
-        fsInputs = {uid: i for uid, i in inputs.items() if droputils.has_path(i)}
-        fsOutputs = {uid: o for uid, o in outputs.items() if droputils.has_path(o)}
         cmd = droputils.replace_path_placeholders(cmd, fsInputs, fsOutputs)
 
-        dataURLInputs = {
-            uid: i for uid, i in inputs.items() if not droputils.has_path(i)
-        }
-        dataURLOutputs = {
-            uid: o for uid, o in outputs.items() if not droputils.has_path(o)
-        }
-        cmd = droputils.replace_dataurl_placeholders(cmd, dataURLInputs, dataURLOutputs)
+        cmd = droputils.replace_dataurl_placeholders(
+            cmd, dataURLInputs, dataURLOutputs
+        )
 
         # Pass down daliuge-specific information to the subprocesses as environment variables
         env = os.environ.copy()
@@ -269,14 +262,6 @@ class BashShellBase(object):
             env.update({"DLG_SESSION_ID": self._dlg_session.sessionId})
 
         env.update({"DLG_ROOT": utils.getDlgDir()})
-        # # try to change to session directory, else just stay in current
-        # work_dir = utils.getDlgWorkDir()
-        # session_dir = f"{work_dir}/{session_id}"
-        # try:
-        #     os.chdir(session_dir)
-        #     logger.info("Changed to session directory: %s" % session_dir)
-        # except:
-        #     logger.warning("Changing to session directory %s unsuccessful!" % session_dir)
 
         # Wrap everything inside bash
         cmd = ("/bin/bash", "-c", cmd)
@@ -303,18 +288,26 @@ class BashShellBase(object):
             pstdout = b"<piped-out>"
         pcode = process.returncode
         end = time.time()
-        logger.info("Finished in %.3f [s] with exit code %d", (end - start), pcode)
+        logger.info(
+            "Finished in %.3f [s] with exit code %d", (end - start), pcode
+        )
 
-        logger.info("Finished in %.3f [s] with exit code %d", (end - start), pcode)
+        logger.info(
+            "Finished in %.3f [s] with exit code %d", (end - start), pcode
+        )
         self._recompute_data["stdout"] = str(pstdout)
         self._recompute_data["stderr"] = str(pstderr)
         self._recompute_data["status"] = str(pcode)
         if pcode == 0 and logger.isEnabledFor(logging.DEBUG):
             logger.debug(
-                mesage_stdouts("Command finished successfully", pstdout, pstderr)
+                mesage_stdouts(
+                    "Command finished successfully", pstdout, pstderr
+                )
             )
         elif pcode != 0:
-            message = "Command didn't finish successfully (exit code %d)" % (pcode,)
+            message = "Command didn't finish successfully (exit code %d)" % (
+                pcode,
+            )
             logger.error(mesage_stdouts(message, pstdout, pstderr))
             raise Exception(message)
 
@@ -388,28 +381,17 @@ class StreamingInputBashAppBase(BashShellBase, AppDROP):
 # @par EAGLE_START
 # @param category BashShellApp
 # @param tag template
-# @param[in] cparam/command Command//String/readwrite/False//False/
-#     \~English The command to be executed
-# @param[in] cparam/input_redirection Input Redirection//String/readwrite/False//False/
-#     \~English The command line argument that specifies the input into this application
-# @param[in] cparam/output_redirection Output Redirection//String/readwrite/False//False/
-#     \~English The command line argument that specifies the output from this application
-# @param[in] cparam/command_line_arguments Command Line Arguments//String/readwrite/False//False/
-#     \~English Additional command line arguments to be added to the command line to be executed
-# @param[in] cparam/paramValueSeparator Param value separator/ /String/readwrite/False//False/
-#     \~English Separator character(s) between parameters on the command line
-# @param[in] cparam/argumentPrefix Argument prefix/"--"/String/readwrite/False//False/
-#     \~English Prefix to each keyed argument on the command line
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
-#     \~English Is this node the start of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @param command Command//String/ComponentParameter/readwrite//False/False/The command to be executed
+# @param input_redirection Input Redirection//String/ComponentParameter/readwrite//False/False/The command line argument that specifies the input into this application
+# @param output_redirection Output Redirection//String/ComponentParameter/readwrite//False/False/The command line argument that specifies the output from this application
+# @param command_line_arguments Command Line Arguments//String/ComponentParameter/readwrite//False/False/Additional command line arguments to be added to the command line to be executed
+# @param paramValueSeparator Param value separator/ /String/ComponentParameter/readwrite//False/False/Separator character(s) between parameters on the command line
+# @param argumentPrefix Argument prefix/"--"/String/ComponentParameter/readwrite//False/False/Prefix to each keyed argument on the command line
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_start Group start/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the start of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
 # @par EAGLE_END
 class BashShellApp(BashShellBase, BarrierAppDROP):
     """
