@@ -23,11 +23,15 @@
 Module containing an example application that calculates a CRC value
 """
 
-from ..drop import BarrierAppDROP, AppDROP
+from ..apps.app_base import BarrierAppDROP, AppDROP
 from dlg.ddap_protocol import AppDROPStates
 
-from ..drop import BarrierAppDROP, AppDROP
-from ..meta import dlg_component, dlg_batch_input, dlg_batch_output, dlg_streaming_input
+from ..meta import (
+    dlg_component,
+    dlg_batch_input,
+    dlg_batch_output,
+    dlg_streaming_input,
+)
 
 try:
     from crc32c import crc32c as crc32  # @UnusedImport
@@ -45,7 +49,8 @@ class CRCApp(BarrierAppDROP):
 
     component_meta = dlg_component(
         "CRCApp",
-        "A BarrierAppDROP that calculates the " "CRC of the single DROP it consumes",
+        "A BarrierAppDROP that calculates the "
+        "CRC of the single DROP it consumes",
         [dlg_batch_input("binary/*", [])],
         [dlg_batch_output("binary/*", [])],
         [dlg_streaming_input("binary/*")],
@@ -60,7 +65,7 @@ class CRCApp(BarrierAppDROP):
         inputDrop = self.inputs[0]
         outputDrop = self.outputs[0]
 
-        bufsize = 4 * 1024 ** 2
+        bufsize = 4 * 1024**2
         desc = inputDrop.open()
         buf = inputDrop.read(desc, bufsize)
         crc = 0
@@ -81,19 +86,13 @@ class CRCApp(BarrierAppDROP):
 # @par EAGLE_START
 # @param category PythonApp
 # @param tag daliuge
-# @param[in] cparam/appclass Application Class/dlg.apps.crc.CRCStreamApp/String/readonly/False//False/
-#     \~English Application class
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
-#     \~English Is this node the start of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
-# @param[out] port/data Data/String/
+# @param appclass Application Class/dlg.apps.crc.CRCStreamApp/String/ComponentParameter/readonly//False/False/Application class
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_start Group start/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the start of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
+# @param data Data//String/OutputPort/readwrite//False/False/Input data stream
 # @par EAGLE_END
 class CRCStreamApp(AppDROP):
     """
