@@ -40,7 +40,8 @@ import docker
 from docker.models.containers import Container
 
 from dlg import utils, droputils
-from dlg.drop import BarrierAppDROP
+from dlg.named_port_utils import replace_named_ports
+from dlg.apps.app_base import BarrierAppDROP
 from dlg.exceptions import DaliugeException, InvalidDropException
 
 
@@ -78,44 +79,25 @@ class ContainerIpWaiter(object):
 # @par EAGLE_START
 # @param category Docker
 # @param tag template
-# @param[in] cparam/image Image//String/readwrite/False//False/
-#     \~English The name of the docker image to be used for this application
-# @param[in] cparam/tag Tag/1.0/String/readwrite/False//False/
-#     \~English The tag of the docker image to be used for this application
-# @param[in] cparam/digest Digest//String/readwrite/False//False/
-#     \~English The hexadecimal hash (long version) of the docker image to be used for this application
-# @param[in] cparam/command Command//String/readwrite/False//False/
-#     \~English The command line to run within the docker instance. The specified command will be executed in a bash shell. That means that images will need a bash shell.
-# @param[in] cparam/input_redirection Input Redirection//String/readwrite/False//False/
-#     \~English The command line argument that specifies the input into this application
-# @param[in] cparam/output_redirection Output Redirection//String/readwrite/False//False/
-#     \~English The command line argument that specifies the output from this application
-# @param[in] cparam/command_line_arguments Command Line Arguments//String/readwrite/False//False/
-#     \~English Additional command line arguments to be added to the command line to be executed
-# @param[in] cparam/paramValueSeparator Param value separator/ /String/readwrite/False//False/
-#     \~English Separator character(s) between parameters and their respective values on the command line
-# @param[in] cparam/argumentPrefix Argument prefix/"--"/String/readwrite/False//False/
-#     \~English Prefix to each keyed argument on the command line
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
-#     \~English Is this node the start of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
-# @param[in] cparam/user User//String/readwrite/False//False/
-#     \~English Username of the user who will run the application within the docker image
-# @param[in] cparam/ensureUserAndSwitch Ensure User And Switch/False/Boolean/readwrite/False//False/
-#     \~English Make sure the user specified in the User parameter exists and then run the docker container as that user
-# @param[in] cparam/removeContainer Remove Container/True/Boolean/readwrite/False//False/
-#     \~English Instruct Docker engine to delete the container after execution is complete
-# @param[in] cparam/additionalBindings Additional Bindings//String/readwrite/False//False/
-#     \~English Directories which will be visible inside the container during run-time. Format is srcdir_on_host:trgtdir_on_container. Multiple entries can be separated by commas.
-# @param[in] cparam/portMappings Port Mappings//String/readwrite/False//False/
-#     \~English Port mappings on the host machine
+# @param image Image//String/ComponentParameter/readwrite//False/False/The name of the docker image to be used for this application
+# @param tag Tag/1.0/String/ComponentParameter/readwrite//False/False/The tag of the docker image to be used for this application
+# @param digest Digest//String/ComponentParameter/readwrite//False/False/The hexadecimal hash (long version) of the docker image to be used for this application
+# @param command Command//String/ComponentParameter/readwrite//False/False/The command line to run within the docker instance. The specified command will be executed in a bash shell. That means that images will need a bash shell.
+# @param input_redirection Input Redirection//String/ComponentParameter/readwrite//False/False/The command line argument that specifies the input into this application
+# @param output_redirection Output Redirection//String/ComponentParameter/readwrite//False/False/The command line argument that specifies the output from this application
+# @param command_line_arguments Command Line Arguments//String/ComponentParameter/readwrite//False/False/Additional command line arguments to be added to the command line to be executed
+# @param paramValueSeparator Param value separator/ /String/ComponentParameter/readwrite//False/False/Separator character(s) between parameters and their respective values on the command line
+# @param argumentPrefix Argument prefix/"--"/String/ComponentParameter/readwrite//False/False/Prefix to each keyed argument on the command line
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_start Group start/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the start of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
+# @param user User//String/ComponentParameter/readwrite//False/False/Username of the user who will run the application within the docker image
+# @param ensureUserAndSwitch Ensure User And Switch/False/Boolean/ComponentParameter/readwrite//False/False/Make sure the user specified in the User parameter exists and then run the docker container as that user
+# @param removeContainer Remove Container/True/Boolean/ComponentParameter/readwrite//False/False/Instruct Docker engine to delete the container after execution is complete
+# @param additionalBindings Additional Bindings//String/ComponentParameter/readwrite//False/False/Directories which will be visible inside the container during run-time. Format is srcdir_on_host:trgtdir_on_container. Multiple entries can be separated by commas.
+# @param portMappings Port Mappings//String/ComponentParameter/readwrite//False/False/Port mappings on the host machine
 # @par EAGLE_END
 class DockerApp(BarrierAppDROP):
     """
@@ -255,12 +237,14 @@ class DockerApp(BarrierAppDROP):
         self._containerLock = multiprocessing.Lock()
         super().initialize(**kwargs)
 
-        self._image = self._getArg(kwargs, "image", None)
-        self._env = self._getArg(kwargs, "env", None)
-        self._cmdLineArgs = self._getArg(kwargs, "command_line_arguments", "")
-        self._applicationArgs = self._getArg(kwargs, "applicationArgs", {})
-        self._argumentPrefix = self._getArg(kwargs, "argumentPrefix", "--")
-        self._paramValueSeparator = self._getArg(kwargs, "paramValueSeparator", " ")
+        self._image = self._popArg(kwargs, "image", None)
+        self._env = self._popArg(kwargs, "env", None)
+        self._cmdLineArgs = self._popArg(kwargs, "command_line_arguments", "")
+        self._applicationArgs = self._popArg(kwargs, "applicationArgs", {})
+        self._argumentPrefix = self._popArg(kwargs, "argumentPrefix", "--")
+        self._paramValueSeparator = self._popArg(
+            kwargs, "paramValueSeparator", " "
+        )
         if not self._image:
             raise InvalidDropException(
                 self, "No docker image specified, cannot create DockerApp"
@@ -273,28 +257,21 @@ class DockerApp(BarrierAppDROP):
                 self._image,
             )
 
-        self._command = self._getArg(kwargs, "command", None)
+        self._command = self._popArg(kwargs, "command", None)
 
         self._noBash = False
         if not self._command or self._command.strip()[:2] == "%%":
-            logger.warning("Assume a default command is executed in the container")
-            self._command = self._command.strip().strip()[:2] if self._command else ""
+            logger.warning(
+                "Assume a default command is executed in the container"
+            )
+            self._command = (
+                self._command.strip().strip()[:2] if self._command else ""
+            )
             self._noBash = True
             # This makes sure that we can retain any command defined in the image, but still be
             # able to add any arguments straight after. This requires to use the placeholder string
             # "%%" at the start of the command, else it is interpreted as a normal command.
 
-        # construct the actual command line from all application parameters
-        self.pargs, self.keyargs = droputils.serialize_applicationArgs(
-            self._applicationArgs, self._argumentPrefix, 
-            self._paramValueSeparator
-        )
-        # defer construction of complete command to run method
-        # cmd = f"{self._command} {self._cmdLineArgs} {' '.join(self.pargs)} "+\
-        #     f"{' '.join(self.keyargs)}"
-        # cmd = cmd.strip()
-        # self._command = cmd
-
         # The user used to run the process in the docker container is now always the user
         # who originally started the DALiuGE process as well. The information is passed through
         # from the host to the engine container (if run as docker) and then further to any
@@ -312,14 +289,14 @@ class DockerApp(BarrierAppDROP):
         # might want to preserve them.
         # TODO: This might be something that the data lifecycle manager could
         # handle, but for the time being we do it here
-        self._removeContainer = self._getArg(kwargs, "removeContainer", True)
+        self._removeContainer = self._popArg(kwargs, "removeContainer", True)
 
         # Ports - a comma seperated list of the host port mappings of form:
         # "hostport1:containerport1, hostport2:containerport2"
-        self._portMappings = self._getArg(kwargs, "portMappings", "")
+        self._portMappings = self._popArg(kwargs, "portMappings", "")
         logger.info(f"portMappings: {self._portMappings}")
 
-        self._shmSize = self._getArg(kwargs, "shmSize", "")
+        self._shmSize = self._popArg(kwargs, "shmSize", "")
         logger.info(f"shmSize: {self._shmSize}")
 
         # Additional volume bindings can be specified for existing files/dirs
@@ -331,7 +308,7 @@ class DockerApp(BarrierAppDROP):
             f"{utils.getDlgDir()}/workspace/settings/passwd:/etc/passwd",
             f"{utils.getDlgDir()}/workspace/settings/group:/etc/group",
         ]
-        additionalBindings = self._getArg(kwargs, "additionalBindings", [])
+        additionalBindings = self._popArg(kwargs, "additionalBindings", [])
         additionalBindings = (
             additionalBindings.split(",")
             if isinstance(additionalBindings, str)
@@ -367,7 +344,9 @@ class DockerApp(BarrierAppDROP):
             start = time.time()
             c.images.pull(self._image)
             end = time.time()
-            logger.debug("Took %.2f [s] to pull image '%s'", (end - start), self._image)
+            logger.debug(
+                "Took %.2f [s] to pull image '%s'", (end - start), self._image
+            )
         else:
             logger.debug("Image '%s' found, no need to pull it", self._image)
 
@@ -376,12 +355,20 @@ class DockerApp(BarrierAppDROP):
         # If none is provided use the session directory
         inspection = c.api.inspect_image(self._image)
         logger.debug("Docker Image inspection: %r", inspection)
-        self.workdir = inspection.get("ContainerConfig", {}).get("WorkingDir", None)
+        self.workdir = inspection.get("ContainerConfig", {}).get(
+            "WorkingDir", None
+        )
         # self.workdir = None
-        self._sessionId = self._dlg_session.sessionId if self._dlg_session else ""
+        self._sessionId = (
+            self._dlg_session.sessionId if self._dlg_session else ""
+        )
         if not self.workdir:
-            default_workingdir = os.path.join(utils.getDlgWorkDir(), self._sessionId)
-            self.workdir = self._getArg(kwargs, "workingDir", default_workingdir)
+            default_workingdir = os.path.join(
+                utils.getDlgWorkDir(), self._sessionId
+            )
+            self.workdir = self._popArg(
+                kwargs, "workingDir", default_workingdir
+            )
 
         c.api.close()
 
@@ -430,7 +417,7 @@ class DockerApp(BarrierAppDROP):
             fsInputs = {uid: i for uid, i in iitems if droputils.has_path(i)}
             fsOutputs = {uid: o for uid, o in oitems if droputils.has_path(o)}
             dockerInputs = {
-                #            uid: DockerPath(utils.getDlgDir() + i.path) for uid, i in fsInputs.items()
+                # uid: DockerPath(utils.getDlgDir() + i.path) for uid, i in fsInputs.items()
                 uid: DockerPath(i.path)
                 for uid, i in fsInputs.items()
             }
@@ -439,42 +426,41 @@ class DockerApp(BarrierAppDROP):
                 uid: DockerPath(o.path)
                 for uid, o in fsOutputs.items()
             }
-            dataURLInputs = {uid: i for uid, i in iitems if not droputils.has_path(i)}
-            dataURLOutputs = {uid: o for uid, o in oitems if not droputils.has_path(o)}
-            self._command = f"{self._command} {' '.join(self.pargs)} "+\
-                f"{' '.join(self.keyargs)}"
-            # TODO: Deal with named inputs
-            if self._command:
-                cmd = droputils.replace_path_placeholders(
-                    self._command, dockerInputs, dockerOutputs
-                )
-                cmd = droputils.replace_dataurl_placeholders(
-                    cmd, dataURLInputs, dataURLOutputs
-                )
-            else:
-                cmd = ""
+            dataURLInputs = {
+                uid: i for uid, i in iitems if not droputils.has_path(i)
+            }
+            dataURLOutputs = {
+                uid: o for uid, o in oitems if not droputils.has_path(o)
+            }
 
             # We bind the inputs and outputs inside the docker under the utils.getDlgDir()
             # directory, maintaining the rest of their original paths.
             # Outputs are bound only up to their dirname (see class doc for details)
             # Volume bindings are setup for FileDROPs and DirectoryContainers only
             binds = [
-                i.path + ":" + dockerInputs[uid].path for uid, i in fsInputs.items()
+                i.path + ":" + dockerInputs[uid].path
+                for uid, i in fsInputs.items()
             ]
             binds += [
-                os.path.dirname(o.path) + ":" + os.path.dirname(dockerOutputs[uid].path)
+                os.path.dirname(o.path)
+                + ":"
+                + os.path.dirname(dockerOutputs[uid].path)
                 for uid, o in fsOutputs.items()
             ]
             logger.debug("Input/output bindings: %r", binds)
             if (
                 len(self._additionalBindings.items()) > 0
             ):  # else we end up with a ':' in the mounts list
-                logger.debug("Additional bindings: %r", self._additionalBindings)
+                logger.debug(
+                    "Additional bindings: %r", self._additionalBindings
+                )
                 binds += [
                     host_path + ":" + container_path
                     for host_path, container_path in self._additionalBindings.items()
                 ]
-            binds = list(set(binds))  # make this a unique list else docker complains
+            binds = list(
+                set(binds)
+            )  # make this a unique list else docker complains
             try:
                 binds.remove(":")
             except:
@@ -489,7 +475,9 @@ class DockerApp(BarrierAppDROP):
                     else:
                         host_port, container_port = mapping.split(":")
                     if host_port not in portMappings:
-                        logger.debug(f"mapping port {host_port} -> {container_port}")
+                        logger.debug(
+                            f"mapping port {host_port} -> {container_port}"
+                        )
                         portMappings[host_port] = int(container_port)
                     else:
                         raise Exception(
@@ -497,13 +485,6 @@ class DockerApp(BarrierAppDROP):
                         )
             logger.debug(f"port mappings: {portMappings}")
 
-            # Wait until the DockerApps this application runtime depends on have
-            # started, and replace their IP placeholders by the real IPs
-            for waiter in self._waiters:
-                uid, ip = waiter.waitForIp()
-                cmd = cmd.replace("%containerIp[{0}]%".format(uid), ip)
-                logger.debug("Command after IP replacement is: %s", cmd)
-
             # deal with environment variables
             env = {}
             env.update({"DLG_UID": self._uid})
@@ -512,7 +493,9 @@ class DockerApp(BarrierAppDROP):
             if self._user is not None:
                 env.update({"USER": self._user, "DLG_ROOT": utils.getDlgDir()})
             if self._env is not None:
-                logger.debug(f"Found environment variable setting: {self._env}")
+                logger.debug(
+                    f"Found environment variable setting: {self._env}"
+                )
                 if (
                     self._env.lower() == "all"
                 ):  # pass on all environment variables from host
@@ -525,7 +508,9 @@ class DockerApp(BarrierAppDROP):
                             "Ignoring provided environment variables: Format wrong? Check documentation"
                         )
                         addEnv = {}
-                    if isinstance(addEnv, dict):  # if it is a dict populate directly
+                    if isinstance(
+                        addEnv, dict
+                    ):  # if it is a dict populate directly
                         # but replace placeholders first
                         for key in addEnv:
                             value = droputils.replace_path_placeholders(
@@ -547,12 +532,57 @@ class DockerApp(BarrierAppDROP):
                     )
             logger.debug(f"Adding environment variables: {env}")
 
+            # deal with named ports
+            appArgs = self._applicationArgs
+            inport_names = (
+                self.parameters["inputs"]
+                if "inputs" in self.parameters
+                else []
+            )
+            outport_names = (
+                self.parameters["outputs"]
+                if "outputs" in self.parameters
+                else []
+            )
+            keyargs, pargs = replace_named_ports(
+                iitems,
+                oitems,
+                inport_names,
+                outport_names,
+                appArgs,
+                argumentPrefix=self._argumentPrefix,
+                separator=self._paramValueSeparator,
+            )
+
+            argumentString = f"{' '.join(keyargs + pargs)}"
+
+            # complete command including all additional parameters and optional redirects
+            cmd = f"{self._command} {argumentString} {self._cmdLineArgs} "
+            if cmd:
+                cmd = droputils.replace_path_placeholders(
+                    cmd, dockerInputs, dockerOutputs
+                )
+                cmd = droputils.replace_dataurl_placeholders(
+                    cmd, dataURLInputs, dataURLOutputs
+                )
+            else:
+                cmd = ""
+            ###############
+            # Wait until the DockerApps this application runtime depends on have
+            # started, and replace their IP placeholders by the real IPs
+            for waiter in self._waiters:
+                uid, ip = waiter.waitForIp()
+                cmd = cmd.replace("%containerIp[{0}]%".format(uid), ip)
+                logger.debug("Command after IP replacement is: %s", cmd)
+
             # Wrap everything inside bash
             if len(cmd) > 0 and not self._noBash:
                 cmd = '/bin/bash -c "%s"' % (
                     utils.escapeQuotes(cmd, singleQuotes=False)
                 )
-                logger.debug("Command after user creation and wrapping is: %s", cmd)
+                logger.debug(
+                    "Command after user creation and wrapping is: %s", cmd
+                )
             else:
                 logger.debug(
                     "executing container with default cmd and wrapped arguments"
@@ -560,7 +590,9 @@ class DockerApp(BarrierAppDROP):
                 cmd = f"{utils.escapeQuotes(cmd, singleQuotes=False)}"
 
             c = DockerApp._get_client()
-            logger.debug(f"Final user for container: {self._user}:{self._userid}")
+            logger.debug(
+                f"Final user for container: {self._user}:{self._userid}"
+            )
 
             # Create container
             self._container = c.containers.create(  # type: ignore
@@ -611,8 +643,12 @@ class DockerApp(BarrierAppDROP):
             end = time.time()
 
             # Capture output
-            stdout = self.container.logs(stream=False, stdout=True, stderr=False)
-            stderr = self.container.logs(stream=False, stdout=False, stderr=True)
+            stdout = self.container.logs(
+                stream=False, stdout=True, stderr=False
+            )
+            stderr = self.container.logs(
+                stream=False, stdout=False, stderr=True
+            )
             if isinstance(stdout, bytes):
                 stdout = stdout.decode()
                 stderr = stderr.decode()
