@@ -30,7 +30,7 @@ import six
 
 from .. import rpc, utils
 from ..ddap_protocol import AppDROPStates
-from ..drop import AppDROP, BarrierAppDROP
+from ..apps.app_base import AppDROP, BarrierAppDROP
 from ..exceptions import InvalidDropException
 
 logger = logging.getLogger(__name__)
@@ -115,7 +115,11 @@ def _to_c_input(i):
     desc = i.open()
     r = _read_cb_type(functools.partial(_read, desc))
     c_input = CDlgInput(
-        i.uid.encode("utf8"), i.oid.encode("utf8"), i.name.encode("utf8"), i.status, r
+        i.uid.encode("utf8"),
+        i.oid.encode("utf8"),
+        i.name.encode("utf8"),
+        i.status,
+        r,
     )
     return desc, c_input
 
@@ -125,8 +129,8 @@ def _to_c_output(o):
     Convert an output drop into its corresponding C structure
     """
 
-    def _write(_o, buf, n):
-        return _o.write(buf[:n])
+    def _write(_o, buf, n, **kwargs):
+        return _o.write(buf[:n], **kwargs)
 
     w = _write_cb_type(functools.partial(_write, o))
     return CDlgOutput(
@@ -187,11 +191,15 @@ def run(lib, c_app, input_closers):
                 raise result
             if result:
                 raise Exception(
-                    "Invocation of {}:run2 returned with status {}".format(lib, result)
+                    "Invocation of {}:run2 returned with status {}".format(
+                        lib, result
+                    )
                 )
 
         elif lib.run(ctypes.pointer(c_app)):
-            raise Exception("Invocation of %r:run returned with status != 0" % lib)
+            raise Exception(
+                "Invocation of %r:run returned with status != 0" % lib
+            )
     finally:
         for closer in input_closers:
             closer()
@@ -225,7 +233,9 @@ def load_and_init(libname, oid, uid, params):
 
         if not found_one:
             raise InvalidLibrary(
-                "{} doesn't have one of the functions {}".format(libname, functions)
+                "{} doesn't have one of the functions {}".format(
+                    libname, functions
+                )
             )
 
     # Create the initial contents of the C dlg_app_info structure
@@ -267,9 +277,12 @@ def load_and_init(libname, oid, uid, params):
         # We need to keep them in a local variable so when we expose them to
         # the app later on via pointers we still have their contents
         local_params = [
-            (str(k).encode("utf8"), str(v).encode("utf8")) for k, v in params.items()
+            (str(k).encode("utf8"), str(v).encode("utf8"))
+            for k, v in params.items()
         ]
-        logger.debug("Extra parameters passed to application: %r", local_params)
+        logger.debug(
+            "Extra parameters passed to application: %r", local_params
+        )
 
         # Wrap in ctypes
         str_ptr_type = ctypes.POINTER(ctypes.c_char_p)
@@ -350,7 +363,9 @@ class DynlibStreamApp(DynlibAppBase, AppDROP):
         self._c_app.n_inputs += 1
 
     def addStreamingInput(self, streamingInputDrop, back=True):
-        super(DynlibStreamApp, self).addStreamingInput(streamingInputDrop, back)
+        super(DynlibStreamApp, self).addStreamingInput(
+            streamingInputDrop, back
+        )
         self._c_app.n_streaming_inputs += 1
 
     def generate_recompute_data(self):
@@ -367,25 +382,19 @@ class DynlibStreamApp(DynlibAppBase, AppDROP):
 # @par EAGLE_START
 # @param category DynlibApp
 # @param tag template
-# @param[in] cparam/libpath Library Path//String/readwrite/False//False/
-#     \~English The location of the shared object/DLL that implements this application
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
-#     \~English Is this node the start of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @param libpath Library Path//String/ComponentParameter/readwrite//False/False/The location of the shared object/DLL that implements this application
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_start Group start/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the start of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
 # @par EAGLE_END
 class DynlibApp(DynlibAppBase, BarrierAppDROP):
     """Loads a dynamic library into the current process and runs it"""
 
     def initialize(self, **kwargs):
         super(DynlibApp, self).initialize(**kwargs)
-        self.ranks = self._getArg(kwargs, "rank", None)
+        self.ranks = self._popArg(kwargs, "rank", None)
 
     def run(self):
         input_closers = prepare_c_inputs(self._c_app, self.inputs)
@@ -431,7 +440,9 @@ def _do_run_in_proc(queue, libname, oid, uid, params, inputs, outputs):
         client.start()
 
         def setup_drop_proxies(inputs, outputs):
-            to_drop_proxy = lambda x: rpc.DropProxy(client, x[0], x[1], x[2], x[3])
+            to_drop_proxy = lambda x: rpc.DropProxy(
+                client, x[0], x[1], x[2], x[3]
+            )
             inputs = [to_drop_proxy(i) for i in inputs]
             outputs = [to_drop_proxy(o) for o in outputs]
             return inputs, outputs
@@ -465,18 +476,12 @@ def get_from_subprocess(proc, q):
 # @par EAGLE_START
 # @param category DynlibProcApp
 # @param tag template
-# @param[in] cparam/libpath Library Path//String/readwrite/False//False/
-#     \~English The location of the shared object/DLL that implements this application
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
-#     \~English Is this node the start of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @param libpath Library Path//String/ComponentParameter/readwrite//False/False/The location of the shared object/DLL that implements this application
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_start Group start/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the start of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
 # @par EAGLE_END
 class DynlibProcApp(BarrierAppDROP):
     """Loads a dynamic library in a different process and runs it"""
@@ -487,7 +492,7 @@ class DynlibProcApp(BarrierAppDROP):
         if "lib" not in kwargs:
             raise InvalidDropException(self, "library not specified")
         self.libname = kwargs.pop("lib")
-        self.timeout = self._getArg(kwargs, "timeout", 600)  # 10 minutes
+        self.timeout = self._popArg(kwargs, "timeout", 600)  # 10 minutes
         self.app_params = kwargs
         self.proc = None
 
