@@ -26,7 +26,7 @@ import signal
 import subprocess
 import sys
 
-from ..drop import BarrierAppDROP
+from dlg.apps.app_base import BarrierAppDROP
 from ..exceptions import InvalidDropException
 
 logger = logging.getLogger(__name__)
@@ -37,30 +37,18 @@ logger = logging.getLogger(__name__)
 # @par EAGLE_START
 # @param category Mpi
 # @param tag template
-# @param[in] cparam/num_of_procs Num procs/1/Integer/readwrite/False//False/
-#     \~English Number of processes used for this application
-# @param[in] cparam/command Command//String/readwrite/False//False/
-#     \~English The command to be executed
-# @param[in] cparam/input_redirection Input Redirection//String/readwrite/False//False/
-#     \~English The command line argument that specifies the input into this application
-# @param[in] cparam/output_redirection Output Redirection//String/readwrite/False//False/
-#     \~English The command line argument that specifies the output from this application
-# @param[in] cparam/command_line_arguments Command Line Arguments//String/readwrite/False//False/
-#     \~English Additional command line arguments to be added to the command line to be executed
-# @param[in] cparam/paramValueSeparator Param value separator/ /String/readwrite/False//False/
-#     \~English Separator character(s) between parameters on the command line
-# @param[in] cparam/argumentPrefix Argument prefix/"--"/String/readwrite/False//False/
-#     \~English Prefix to each keyed argument on the command line
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_end Group end/False/Boolean/readwrite/False//False/
-#     \~English Is this node the end of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @param num_of_procs Num procs/1/Integer/ComponentParameter/readwrite//False/False/Number of processes used for this application
+# @param command Command//String/ComponentParameter/readwrite//False/False/The command to be executed
+# @param input_redirection Input Redirection//String/ComponentParameter/readwrite//False/False/The command line argument that specifies the input into this application
+# @param output_redirection Output Redirection//String/ComponentParameter/readwrite//False/False/The command line argument that specifies the output from this application
+# @param command_line_arguments Command Line Arguments//String/ComponentParameter/readwrite//False/False/Additional command line arguments to be added to the command line to be executed
+# @param paramValueSeparator Param value separator/ /String/ComponentParameter/readwrite//False/False/Separator character(s) between parameters on the command line
+# @param argumentPrefix Argument prefix/"--"/String/ComponentParameter/readwrite//False/False/Prefix to each keyed argument on the command line
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_end Group end/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the end of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
 # @par EAGLE_END
 class MPIApp(BarrierAppDROP):
     """
@@ -78,10 +66,10 @@ class MPIApp(BarrierAppDROP):
     def initialize(self, **kwargs):
         super(MPIApp, self).initialize(**kwargs)
 
-        self._command = self._getArg(kwargs, "command", None)
-        self._maxprocs = self._getArg(kwargs, "maxprocs", 1)
-        self._use_wrapper = self._getArg(kwargs, "use_wrapper", False)
-        self._args = self._getArg(kwargs, "args", [])
+        self._command = self._popArg(kwargs, "command", None)
+        self._maxprocs = self._popArg(kwargs, "maxprocs", 1)
+        self._use_wrapper = self._popArg(kwargs, "use_wrapper", False)
+        self._args = self._popArg(kwargs, "args", [])
         if not self._command:
             raise InvalidDropException(
                 self, "No command specified, cannot create MPIApp"
@@ -115,25 +103,39 @@ class MPIApp(BarrierAppDROP):
         vendor, version = MPI.get_vendor()  # @UndefinedVariable
         info = MPI.Info.Create()  # @UndefinedVariable
         logger.debug(
-            "MPI vendor is %s, version %s", vendor, ".".join([str(x) for x in version])
+            "MPI vendor is %s, version %s",
+            vendor,
+            ".".join([str(x) for x in version]),
         )  # @UndefinedVariable
         comm_children = MPI.COMM_SELF.Spawn(
-            cmd, args=args, maxprocs=self._maxprocs, errcodes=errcodes, info=info
+            cmd,
+            args=args,
+            maxprocs=self._maxprocs,
+            errcodes=errcodes,
+            info=info,
         )  # @UndefinedVariable
 
         n_children = comm_children.Get_remote_size()
-        logger.info("%d MPI children apps spawned, gathering exit data", n_children)
+        logger.info(
+            "%d MPI children apps spawned, gathering exit data", n_children
+        )
 
         if self._use_wrapper:
             children_data = comm_children.gather(
                 ("", "", 0), root=MPI.ROOT
             )  # @UndefinedVariable
             exit_codes = [x[2] for x in children_data]
-            logger.info("Exit codes gathered from children processes: %r", exit_codes)
+            logger.info(
+                "Exit codes gathered from children processes: %r", exit_codes
+            )
 
             any_failed = False
             for rank, (stdout, stderr, code) in enumerate(children_data):
-                self._recompute_data[str(rank)] = [code, str(stdout), str(stderr)]
+                self._recompute_data[str(rank)] = [
+                    code,
+                    str(stdout),
+                    str(stderr),
+                ]
                 if code == 0:
                     continue
                 any_failed = True
