@@ -27,7 +27,9 @@ import collections
 from enum import Enum
 import importlib
 import inspect
+import json
 import logging
+import os
 import pickle
 
 from typing import Callable
@@ -36,7 +38,8 @@ from io import StringIO
 from contextlib import redirect_stdout
 
 from dlg import droputils, utils
-from dlg.drop import BarrierAppDROP
+from dlg.named_port_utils import check_ports_dict, identify_named_ports
+from dlg.apps.app_base import BarrierAppDROP
 from dlg.exceptions import InvalidDropException
 from dlg.meta import (
     dlg_string_param,
@@ -60,7 +63,6 @@ def deserialize_data(d):
 
 
 def serialize_func(f):
-
     if isinstance(f, str):
         parts = f.split(".")
         f = getattr(importlib.import_module(".".join(parts[:-1])), parts[-1])
@@ -71,7 +73,10 @@ def serialize_func(f):
     a = inspect.getfullargspec(f)
     if a.defaults:
         fdefaults["kwargs"] = dict(
-            zip(a.args[-len(a.defaults) :], [serialize_data(d) for d in a.defaults])
+            zip(
+                a.args[-len(a.defaults) :],
+                [serialize_data(d) for d in a.defaults],
+            )
         )
         adefaults["kwargs"] = dict(
             zip(a.args[-len(a.defaults) :], [d for d in a.defaults])
@@ -82,29 +87,53 @@ def serialize_func(f):
 
 
 def import_using_name(app, fname):
-    # If only one part check if builtin
+    logger.debug("Import from %s", fname)
     parts = fname.split(".")
+    # If only one part check if builtin
     if len(parts) < 2:
-        b = globals()['__builtins__']
+        b = globals()["__builtins__"]
         logger.debug(f"Builtins: {type(b)}")
         logger.debug(f"Function {fname}: {hasattr(b, fname)}")
         if fname in b:
             return b[fname]
         else:
-            msg = "%s is not builtin and does not contain a module name" % fname
+            msg = (
+                "%s is not builtin and does not contain a module name" % fname
+            )
             raise InvalidDropException(app, msg)
     else:
-        modname, fname = ".".join(parts[:-1]), parts[-1]
-        try:
-            mod = importlib.import_module(modname, __name__)
-            return getattr(mod, fname)
-        except ImportError as e:
-            raise InvalidDropException(
-                app, "Error when loading module %s: %s" % (modname, str(e))
-            )
-        except AttributeError:
-            raise InvalidDropException(app, "Module %s has no member %s" % (modname, fname))
+        if len(parts) > 1:
+            if parts[-1] in ["__init__", "__class__"]:
+                parts = parts[:-1]
+            logger.debug("Recursive import: %s", parts)
+            try:
+                mod = importlib.import_module(parts[0], __name__)
+            except ImportError as e:
+                raise InvalidDropException(
+                    app,
+                    "Error when loading module %s: %s" % (parts[0], str(e)),
+                )
+            for m in parts[1:]:
+                try:
+                    logger.debug("Getting attribute %s", m)
+                    mod = getattr(mod, m)
+                except AttributeError as e:
+                    try:
+                        logger.debug(
+                            "Trying to load backwards: %s",
+                            ".".join(parts[:-1]),
+                        )
+                        mod = importlib.import_module(
+                            ".".join(parts[:-1]), __name__
+                        )
+                        mod = getattr(mod, parts[-1])
+                        break
+                    except Exception as e:
+                        raise InvalidDropException(
+                            app, "Problem importing module %s, %s" % (mod, e)
+                        )
 
+            return mod
 
 
 def import_using_code(code):
@@ -112,12 +141,13 @@ def import_using_code(code):
 
 
 class DropParser(Enum):
-    PICKLE = 'pickle'
-    EVAL = 'eval'
-    NPY = 'npy'
-    #JSON = "json"
-    PATH = 'path'  # input only
-    DATAURL = 'dataurl'  # input only
+    PICKLE = "pickle"
+    EVAL = "eval"
+    NPY = "npy"
+    # JSON = "json"
+    PATH = "path"  # input only
+    DATAURL = "dataurl"  # input only
+
 
 ##
 # @brief PyFuncApp
@@ -130,30 +160,20 @@ class DropParser(Enum):
 # @par EAGLE_START
 # @param category PythonApp
 # @param tag template
-# @param[in] cparam/appclass Application Class/dlg.apps.pyfunc.PyFuncApp/String/readonly/False//False/
-#     \~English Application class
-# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
-#     \~English Estimated execution time
-# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
-#     \~English Number of cores used
-# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
-#     \~English Is this node the start of a group?
-# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
-#     \~English The allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
-# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
-#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
-# @param[in] aparam/func_name Function Name//String/readwrite/False//False/
-#     \~English Python function name
-# @param[in] aparam/func_code Function Code//String/readwrite/False//False/
-#     \~English Python function code, e.g. 'def function_name(args): return args'
-# @param[in] aparam/input_parser Input Parser/pickle/Select/readwrite/False/pickle,eval,npy,path,dataurl/False/
-#     \~English Input port parsing technique
-# @param[in] aparam/output_parser Output Parser/pickle/Select/readwrite/False/pickle,eval,npy/False/
-#     \~English output port parsing technique
-# @param[in] aparam/func_defaults Function Defaults//String/readwrite/False//False/
+# @param appclass Application Class/dlg.apps.pyfunc.PyFuncApp/String/ComponentParameter/readonly//False/False/Application class
+# @param execution_time Execution Time/5/Float/ComponentParameter/readonly//False/False/Estimated execution time
+# @param num_cpus No. of CPUs/1/Integer/ComponentParameter/readonly//False/False/Number of cores used
+# @param group_start Group start/False/Boolean/ComponentParameter/readwrite//False/False/Is this node the start of a group?
+# @param input_error_threshold "Input error rate (%)"/0/Integer/ComponentParameter/readwrite//False/False/The allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param n_tries Number of tries/1/Integer/ComponentParameter/readwrite//False/False/Specifies the number of times the 'run' method will be executed before finally giving up
+# @param func_name Function Name//String/ApplicationArgument/readwrite//False/False/Python function name
+# @param func_code Function Code//String/ApplicationArgument/readwrite//False/False/Python function code, e.g. 'def function_name(args): return args'
+# @param input_parser Input Parser/pickle/Select/ApplicationArgument/readwrite/pickle,eval,npy,path,dataurl/False/False/Input port parsing technique
+# @param output_parser Output Parser/pickle/Select/ApplicationArgument/readwrite/pickle,eval,npy/False/False/Output port parsing technique
+# @param func_defaults Function Defaults//String/ApplicationArgument/readwrite//False/False/
 #     \~English Mapping from argname to default value. Should match only the last part of the argnames list.
 #               Values are interpreted as Python code literals and that means string values need to be quoted.
-# @param[in] aparam/func_arg_mapping Function Arguments Mapping//String/readwrite/False//False/
+# @param func_arg_mapping Function Arguments Mapping//String/ApplicationArgument/readwrite//False/False/
 #     \~English Mapping between argument name and input drop uids
 # @par EAGLE_END
 class PyFuncApp(BarrierAppDROP):
@@ -216,15 +236,40 @@ class PyFuncApp(BarrierAppDROP):
         Inititalize self.func_defaults dictionary from values provided.
         Multiple options exist and some are here for compatibility.
         """
-        logger.debug(f"Starting evaluation of func_defaults: {self.func_defaults}")
+        logger.debug(
+            f"Starting evaluation of func_defaults: {self.func_defaults}"
+        )
+        self.arguments = inspect.getfullargspec(self.f)
+        self.argsig = inspect.signature(
+            self.f
+        )  # TODO: Move to using signature only
+        logger.debug("Function inspection revealed %s", self.arguments)
+        logger.debug("Function sigature: %s", self.argsig)
+        self.fn_nargs = len(self.arguments.args)
+        self.fn_npos = sum(
+            [
+                1
+                if p.kind == p.POSITIONAL_OR_KEYWORD
+                and p.default != inspect._empty
+                else 0
+                for p in self.argsig.parameters.values()
+            ]
+        )
+        self.arguments_defaults = [
+            v.default if v.default != inspect._empty else None
+            for v in self.argsig.parameters.values()
+        ]
+        logger.debug("Got signature for function %s %s", self.f, self.argsig)
+        self.fn_defaults = self.arguments_defaults
+
+        self.fn_ndef = (
+            len(self.arguments_defaults) if self.arguments_defaults else 0
+        )
         if (
             isinstance(self.func_defaults, dict)
             and len(self.func_defaults) > 0
             and list(self.func_defaults.keys()) == ["kwargs", "args"]
         ):
-            # we bring everything back to just kwargs, because positional args are messy
-            # NOTE: This means that positional ONLY arguments won't work, but those are not used
-            # too often.
             for arg in self.func_defaults["args"]:
                 self.func_defaults["kwargs"][arg] = arg
                 self.func_defaults = self.func_defaults["kwargs"]
@@ -238,7 +283,9 @@ class PyFuncApp(BarrierAppDROP):
         if not isinstance(self.func_defaults, dict):
             logger.error(
                 "Wrong format or type for function defaults for %s: %r, %r",
-                self.f.__name__, self.func_defaults, type(self.func_defaults)
+                self.f.__name__,
+                self.func_defaults,
+                type(self.func_defaults),
             )
             raise ValueError
         if self.input_parser is DropParser.PICKLE:
@@ -248,14 +295,22 @@ class PyFuncApp(BarrierAppDROP):
 
         # set the function defaults from introspection
         if self.arguments:
-            self.fn_npos = len(self.arguments.args) - self.fn_ndef
+            # self.fn_npos = len(self.arguments.args) - self.fn_ndef
+            self.fn_posargs = self.arguments.args[
+                : self.fn_npos
+            ]  # positional arg names
             self.fn_defaults = {
                 name: None for name in self.arguments.args[: self.fn_npos]
             }
             logger.debug(f"initialized fn_defaults with {self.fn_defaults}")
             # deal with args and kwargs
             kwargs = (
-                dict(zip(self.arguments.args[self.fn_npos :], self.arguments.defaults))
+                dict(
+                    zip(
+                        self.arguments.args[self.fn_npos :],
+                        self.arguments.defaults,
+                    )
+                )
                 if self.arguments.defaults
                 else {}
             )
@@ -264,26 +319,31 @@ class PyFuncApp(BarrierAppDROP):
             # deal with kwonlyargs
             if self.arguments.kwonlydefaults:
                 kwonlyargs = dict(
-                    zip(self.arguments.kwonlyargs, self.arguments.kwonlydefaults)
+                    zip(
+                        self.arguments.kwonlyargs,
+                        self.arguments.kwonlydefaults,
+                    )
                 )
                 self.fn_defaults.update(kwonlyargs)
                 logger.debug(f"fn_defaults updated with {kwonlyargs}")
 
-            self.fn_posargs = self.arguments.args[
-                : self.fn_npos
-            ]  # positional arg names
-
     def initialize(self, **kwargs):
         """
         The initialization of a function component is mainly dealing with mapping
         inputs and provided applicationArgs to the function arguments. All of this
-        should be driven by matching names, but currently that is not being done.
+        should be driven by matching names.
         """
         BarrierAppDROP.initialize(self, **kwargs)
 
-        self._applicationArgs = self._getArg(kwargs, "applicationArgs", {})
+        env = os.environ.copy()
+        env.update({"DLG_UID": self._uid})
+        if self._dlg_session:
+            env.update({"DLG_SESSION_ID": self._dlg_session.sessionId})
 
-        self.func_code = self._getArg(kwargs, "func_code", None)
+        self._applicationArgs = self._popArg(kwargs, "applicationArgs", {})
+
+        self.func_code = self._popArg(kwargs, "func_code", None)
+        self.arguments_defaults = []
 
         # check for function definition arguments in applicationArgs
         self.func_def_keywords = [
@@ -294,7 +354,7 @@ class PyFuncApp(BarrierAppDROP):
             "output_parser",
             "func_defaults",
             "pickle",
-            ]
+        ]
 
         # backwards compatibility
         if "pickle" in self._applicationArgs:
@@ -309,9 +369,10 @@ class PyFuncApp(BarrierAppDROP):
         for kw in self.func_def_keywords:
             if kw in self._applicationArgs:  # these are the preferred ones now
                 if isinstance(
-                    self._applicationArgs[kw]["value"], bool
+                    self._applicationArgs[kw]["value"],
+                    bool
                     or self._applicationArgs[kw]["value"]
-                    or self._applicationArgs[kw]["precious"]
+                    or self._applicationArgs[kw]["precious"],
                 ):
                     # only transfer if there is a value or precious is True
                     self._applicationArgs.pop(kw)
@@ -330,7 +391,9 @@ class PyFuncApp(BarrierAppDROP):
             self.f = import_using_name(self, self.func_name)
         else:
             if not isinstance(self.func_code, bytes):
-                self.func_code = base64.b64decode(self.func_code.encode("utf8"))
+                self.func_code = base64.b64decode(
+                    self.func_code.encode("utf8")
+                )
             self.f = import_using_code(self.func_code)
         # make sure defaults are dicts
         if isinstance(self.func_defaults, str):
@@ -338,17 +401,10 @@ class PyFuncApp(BarrierAppDROP):
         if isinstance(self.func_arg_mapping, str):
             self.func_arg_mapping = ast.literal_eval(self.func_arg_mapping)
 
-        self.arguments = inspect.getfullargspec(self.f)
-        logger.debug(f"Function inspection revealed {self.arguments}")
-        # we don't want to mess with the 'self' argument
-        if self.arguments.args.count('self'): 
-            self.arguments.args.remove('self')
-        self.fn_nargs = len(self.arguments.args)
-        self.fn_ndef = len(self.arguments.defaults) if self.arguments.defaults else 0
         self._init_func_defaults()
         logger.info(f"Args summary for '{self.func_name}':")
         logger.info(f"Args: {self.arguments.args}")
-        logger.info(f"Args defaults:  {self.arguments.defaults}")
+        logger.info(f"Args defaults:  {self.fn_defaults}")
         logger.info(f"Args positional: {self.arguments.args[:self.fn_npos]}")
         logger.info(f"Args keyword: {self.arguments.args[self.fn_npos:]}")
         logger.info(f"Args supplied:  {self.func_defaults}")
@@ -356,7 +412,7 @@ class PyFuncApp(BarrierAppDROP):
         logger.info(f"VarKwds allowed:  {self.arguments.varkw}")
 
         # Mapping between argument name and input drop uids
-        logger.debug(f"Input mapping: {self.func_arg_mapping}")
+        logger.debug(f"Input mapping provided: {self.func_arg_mapping}")
         self._recompute_data = {}
 
     def run(self):
@@ -392,14 +448,16 @@ class PyFuncApp(BarrierAppDROP):
         # Inputs are un-pickled and treated as the arguments of the function
         # Their order must be preserved, so we use an OrderedDict
         if self.input_parser is DropParser.PICKLE:
-            #all_contents = lambda x: pickle.loads(droputils.allDropContents(x))
+            # all_contents = lambda x: pickle.loads(droputils.allDropContents(x))
             all_contents = droputils.load_pickle
         elif self.input_parser is DropParser.EVAL:
+
             def optionalEval(x):
                 # Null and Empty Drops will return an empty byte string
                 # which should propogate back to None
-                content: str = droputils.allDropContents(x).decode('utf-8')
+                content: str = droputils.allDropContents(x).decode("utf-8")
                 return ast.literal_eval(content) if len(content) > 0 else None
+
             all_contents = optionalEval
         elif self.input_parser is DropParser.NPY:
             all_contents = droputils.load_npy
@@ -416,8 +474,11 @@ class PyFuncApp(BarrierAppDROP):
 
         outputs = collections.OrderedDict()
         for uid, drop in self._outputs.items():
-            outputs[uid] = all_contents(drop) if self.output_parser is DropParser.PATH else None
-
+            outputs[uid] = (
+                all_contents(drop)
+                if self.output_parser is DropParser.PATH
+                else None
+            )
 
         # Keyword arguments are made up of the default values plus the inputs
         # that match one of the keyword argument names
@@ -441,102 +502,149 @@ class PyFuncApp(BarrierAppDROP):
         # if we have named ports use the inputs with
         # the correct UIDs
         logger.debug(f"Parameters found: {self.parameters}")
-        posargs = self.arguments.args[:self.fn_npos]
+        posargs = self.arguments.args[: self.fn_npos]
+        keyargs = self.arguments.args[self.fn_npos :]
         kwargs = {}
         pargs = []
         # Initialize pargs dictionary and update with provided argument values
-        pargsDict = collections.OrderedDict(zip(posargs,[None]*len(posargs)))
+        pargsDict = collections.OrderedDict(
+            zip(posargs, [None] * len(posargs))
+        )
+        if self.arguments_defaults:
+            keyargsDict = dict(
+                zip(keyargs, self.arguments_defaults[self.fn_npos :])
+            )
+        else:
+            keyargsDict = {}
+        logger.debug("Initial keyargs dictionary: %s", keyargsDict)
         if "applicationArgs" in self.parameters:
-            appArgs = self.parameters["applicationArgs"]  # we'll pop the identified ones
-            pargsDict.update({k:self.parameters[k] for k in pargsDict if k in 
-                self.parameters})
+            appArgs = self.parameters[
+                "applicationArgs"
+            ]  # we'll pop the identified ones
+            _dum = [
+                appArgs.pop(k) for k in self.func_def_keywords if k in appArgs
+            ]
+            logger.debug(
+                "Identified keyword arguments removed: %s",
+                [i["text"] for i in _dum],
+            )
+            pargsDict.update(
+                {
+                    k: self.parameters[k]
+                    for k in pargsDict
+                    if k in self.parameters
+                }
+            )
             # if defined in both we use AppArgs values
-            pargsDict.update({k:appArgs[k]['value'] for k in pargsDict if k 
-                in appArgs})
+            pargsDict.update(
+                {k: appArgs[k]["value"] for k in pargsDict if k in appArgs}
+            )
             logger.debug("Initial posargs dictionary: %s", pargsDict)
         else:
             appArgs = {}
 
-        if ('inputs' in self.parameters and isinstance(self.parameters['inputs'][0], dict)):
-            check_len = min(len(inputs),self.fn_nargs+
-                len(self.arguments.kwonlyargs))
-            kwargs.update(droputils.identify_named_ports(
-                inputs,
-                self.parameters['inputs'],
-                posargs,
-                pargsDict,
-                appArgs,
-                check_len=check_len,
-                mode="inputs"))
+        if "inputs" in self.parameters and check_ports_dict(
+            self.parameters["inputs"]
+        ):
+            check_len = min(
+                len(inputs), self.fn_nargs + len(self.arguments.kwonlyargs)
+            )
+            inputs_dict = collections.OrderedDict()
+            for inport in self.parameters["inputs"]:
+                key = list(inport.keys())[0]
+                inputs_dict[key] = {"name": inport[key], "path": inputs[key]}
+            kwargs.update(
+                identify_named_ports(
+                    inputs_dict,
+                    posargs,
+                    pargsDict,
+                    keyargsDict,
+                    check_len=check_len,
+                    mode="inputs",
+                )
+            )
         else:
-            for i in range(min(len(inputs),self.fn_nargs)):
-                kwargs.update({self.arguments.args[i]: list(inputs.values())[i]})
+            for i in range(min(len(inputs), self.fn_nargs)):
+                kwargs.update(
+                    {self.arguments.args[i]: list(inputs.values())[i]}
+                )
 
+        if "outputs" in self.parameters and check_ports_dict(
+            self.parameters["outputs"]
+        ):
+            check_len = min(
+                len(outputs), self.fn_nargs + len(self.arguments.kwonlyargs)
+            )
+            outputs_dict = collections.OrderedDict()
+            for outport in self.parameters["outputs"]:
+                key = list(outport.keys())[0]
+                outputs_dict[key] = {
+                    "name": outport[key],
+                    "path": outputs[key],
+                }
+
+            kwargs.update(
+                identify_named_ports(
+                    outputs_dict,
+                    posargs,
+                    pargsDict,
+                    keyargsDict,
+                    check_len=check_len,
+                    mode="outputs",
+                )
+            )
         logger.debug(f"Updating funcargs with input ports {kwargs}")
         funcargs.update(kwargs)
 
-        if ('outputs' in self.parameters and isinstance(self.parameters['outputs'][0], dict)):
-            check_len = min(len(outputs),self.fn_nargs+
-                len(self.arguments.kwonlyargs))
-            kwargs.update(droputils.identify_named_ports(
-                outputs,
-                self.parameters['outputs'],
-                posargs,
-                pargsDict,
-                appArgs,
-                check_len=check_len,
-                mode="outputs"))
-
         # Try to get values for still missing positional arguments from Application Args
         if "applicationArgs" in self.parameters:
-            _dum = [appArgs.pop(k) for k in self.func_def_keywords if k in appArgs]
-            logger.debug("Identified keyword arguments removed: %s",
-                [i['text'] for i in _dum])
             for pa in posargs:
-                if pa != 'self' and pa not in funcargs:
+                if pa != "self" and pa not in funcargs:
                     if pa in appArgs:
                         arg = appArgs.pop(pa)
-                        value = arg['value']
-                        ptype = arg['type']
+                        value = arg["value"]
+                        ptype = arg["type"]
                         if ptype in ["Complex", "Json"]:
                             try:
                                 value = ast.literal_eval(value)
                             except Exception as e:
                                 # just go on if this did not work
-                                logger.warning("Eval raised an error: %s",e)
+                                logger.warning("Eval raised an error: %s", e)
                         elif ptype in ["Python"]:
                             try:
                                 import numpy
+
                                 value = eval(value, {"numpy": numpy}, {})
                             except:
                                 pass
                         pargsDict.update({pa: value})
-                    elif pa != 'self' and pa not in pargsDict:
-                        logger.warning(f"Required positional argument '{pa}' not found!")
+                    elif pa != "self" and pa not in pargsDict:
+                        logger.warning(
+                            f"Required positional argument '{pa}' not found!"
+                        )
             _dum = [appArgs.pop(k) for k in pargsDict if k in appArgs]
-            logger.debug("Identified positional arguments removed: %s", 
-                [i['text'] for i in _dum])
+            logger.debug(
+                "Identified positional arguments removed: %s",
+                [i["text"] for i in _dum],
+            )
             logger.debug(f"updating posargs with {list(pargsDict.keys())}")
             pargs.extend(list(pargsDict.values()))
 
             # Try to get values for still missing kwargs arguments from Application kws
             kwargs = {}
-            kws = self.arguments.args[self.fn_npos:]
+            kws = self.arguments.args[self.fn_npos :]
             for ka in kws:
                 if ka not in funcargs:
                     if ka in appArgs:
                         arg = appArgs.pop(ka)
-                        value = arg['value']
-                        ptype = arg['type']
+                        value = arg["value"]
+                        ptype = arg["type"]
                         if ptype in ["Complex", "Json"]:
                             try:
                                 value = ast.literal_eval(value)
                             except:
                                 pass
-                        kwargs.update({
-                            ka:
-                            value
-                        })
+                        kwargs.update({ka: value})
                     else:
                         logger.warning(f"Keyword argument '{ka}' not found!")
             logger.debug(f"updating funcargs with {kwargs}")
@@ -549,19 +657,18 @@ class PyFuncApp(BarrierAppDROP):
                 if ka not in funcargs:
                     if ka in appArgs:
                         arg = appArgs.pop(ka)
-                        value = arg['value']
-                        ptype = arg['type']
+                        value = arg["value"]
+                        ptype = arg["type"]
                         if ptype in ["Complex", "Json"]:
                             try:
                                 value = ast.literal_eval(value)
                             except:
                                 pass
-                        kwargs.update({
-                            ka:
-                            value
-                        })
+                        kwargs.update({ka: value})
                     else:
-                        logger.warning(f"Keyword only argument '{ka}' not found!")
+                        logger.warning(
+                            f"Keyword only argument '{ka}' not found!"
+                        )
             logger.debug(f"updating funcargs with kwonlyargs: {kwargs}")
             funcargs.update(kwargs)
 
@@ -570,18 +677,20 @@ class PyFuncApp(BarrierAppDROP):
             vkarg = {}
             logger.debug(f"Remaining AppArguments {appArgs}")
             for arg in appArgs:
-                if appArgs[arg]['type'] in ['Json', 'Complex']:
-                    value = ast.literal_eval(appArgs[arg]['value'])
+                if appArgs[arg]["type"] in ["Json", "Complex"]:
+                    value = ast.literal_eval(appArgs[arg]["value"])
                 else:
-                    value = appArgs[arg]['value']
-                if appArgs[arg]['positional']:
+                    value = appArgs[arg]["value"]
+                if appArgs[arg]["positional"]:
                     vparg.append(value)
                 else:
-                    vkarg.update({arg:value})
+                    vkarg.update({arg: value})
 
             if self.arguments.varargs:
+                logger.debug("Adding remaining posargs to varargs")
                 pargs.extend(vparg)
             if self.arguments.varkw:
+                logger.debug("Adding remaining kwargs to varkw")
                 funcargs.update(vkarg)
 
         # Fill rest with default arguments if there are any more
@@ -599,7 +708,10 @@ class PyFuncApp(BarrierAppDROP):
         capture = StringIO()
         with redirect_stdout(capture):
             result = self.f(*pargs, **funcargs)
-        logger.info(f"Captured output from function app '{self.func_name}': {capture.getvalue()}")
+        logger.debug("Returned result from %s: %s", self.func_name, result)
+        logger.info(
+            f"Captured output from function app '{self.func_name}': {capture.getvalue()}"
+        )
         logger.debug(f"Finished execution of {self.func_name}.")
 
         # Depending on how many outputs we have we treat our result
@@ -617,11 +729,17 @@ class PyFuncApp(BarrierAppDROP):
                     logger.debug(f"Writing pickeled result {type(r)} to {o}")
                     o.write(pickle.dumps(r))
                 elif self.output_parser is DropParser.EVAL:
-                    o.write(repr(r).encode('utf-8'))
+                    o.write(repr(r).encode("utf-8"))
                 elif self.output_parser is DropParser.NPY:
                     droputils.save_npy(o, r)
                 else:
                     ValueError(self.output_parser.__repr__())
 
     def generate_recompute_data(self):
+        for name, val in self._recompute_data.items():
+            try:
+                json.dumps(val)
+            except TypeError as e:
+                logger.debug(e)
+                self._recompute_data[name] = repr(val)
         return self._recompute_data
