@@ -29,11 +29,13 @@ import socket
 import struct
 import time
 
+from dlg.common import CategoryType, DropType
+
 from . import utils, droputils
 from .apps import pyfunc
 from .common import dropdict, Categories
 from .ddap_protocol import DROPStates
-from .drop import BarrierAppDROP
+from .apps.app_base import BarrierAppDROP
 from .exceptions import InvalidDropException
 
 logger = logging.getLogger(__name__)
@@ -45,8 +47,8 @@ class ResultTransmitter(BarrierAppDROP):
 
     def initialize(self, **kwargs):
         BarrierAppDROP.initialize(self, input_error_threshold=100, **kwargs)
-        self.host = self._getArg(kwargs, "host", "127.0.0.1")
-        self.port = self._getArg(kwargs, "port", None)
+        self.host = self._popArg(kwargs, "host", "localhost")
+        self.port = self._popArg(kwargs, "port", None)
         if self.port is None:
             raise InvalidDropException(self, "Missing port parameter")
 
@@ -81,14 +83,18 @@ def _get_client(**kwargs):
     from .manager.client import NodeManagerClient
     from .manager import constants
 
-    host = kwargs.get("host", "127.0.0.1")
+    host = kwargs.get("host", "localhost")
     port = kwargs.get("port", constants.NODE_DEFAULT_REST_PORT)
     timeout = kwargs.get("timeout", None)
     return NodeManagerClient(host, port, timeout)
 
 
 def _is_list_of_delayeds(x):
-    return isinstance(x, (list, tuple)) and len(x) > 0 and isinstance(x[0], _DataDrop)
+    return (
+        isinstance(x, (list, tuple))
+        and len(x) > 0
+        and isinstance(x[0], _DataDrop)
+    )
 
 
 def compute(value, **kwargs):
@@ -107,10 +113,13 @@ def compute(value, **kwargs):
     transmitter = dropdict(
         {
             "type": "app",
+            #            "categoryType": CategoryType.APPLICATION,
             "app": "dlg.dask_emulation.ResultTransmitter",
+            "appclass": "dlg.dask_emulation.ResultTransmitter",
             "oid": transmitter_oid,
             "port": port,
             "nm": "result transmitter",
+            "text": "result transmitter",
         }
     )
     for leaf_oid in droputils.get_leaves(graph.values()):
@@ -124,7 +133,9 @@ def compute(value, **kwargs):
     client = _get_client(**kwargs)
     client.create_session(session_id)
     client.append_graph(session_id, graph)
-    client.deploy_session(session_id, completed_uids=droputils.get_roots(graph))
+    client.deploy_session(
+        session_id, completed_uids=droputils.get_roots(graph)
+    )
 
     timeout = kwargs.get("timeout", None)
     s = utils.connect_to("localhost", port, timeout)
@@ -212,12 +223,20 @@ class _DelayedDrop(object):
         if isinstance(self, _DataDrop):
             self_dd.addProducer(up_dd)
             logger.debug(
-                "Set %r/%s as producer of %r/%s", upstream, upstream.oid, self, self.oid
+                "Set %r/%s as producer of %r/%s",
+                upstream,
+                upstream.oid,
+                self,
+                self.oid,
             )
         else:
             self_dd.addInput(up_dd)
             logger.debug(
-                "Set %r/%s as input of %r/%s", upstream, upstream.oid, self, self.oid
+                "Set %r/%s as input of %r/%s",
+                upstream,
+                upstream.oid,
+                self,
+                self.oid,
             )
 
 
@@ -227,7 +246,10 @@ class _Listifier(BarrierAppDROP):
     def run(self):
         self.outputs[0].write(
             pickle.dumps(
-                [pickle.loads(droputils.allDropContents(x)) for x in self.inputs]
+                [
+                    pickle.loads(droputils.allDropContents(x))
+                    for x in self.inputs
+                ]
             )
         )
 
@@ -265,7 +287,13 @@ class _DelayedDrops(_DelayedDrop):
 
     def make_dropdict(self):
         return dropdict(
-            {"type": "app", "app": "dlg.dask_emulation._Listifier", "nm": "listifier"}
+            {
+                "type": "app",
+                "categoryType": CategoryType.APPLICATION,
+                "app": "dlg.dask_emulation._Listifier",
+                "nm": "listifier",
+                "text": "listifier",
+            }
         )
 
     def __repr__(self):
@@ -291,12 +319,19 @@ class _AppDrop(_DelayedDrop):
         self.kwarg_names = list(self.original_kwarg_names)
         self.kwarg_names.reverse()
         my_dropdict = dropdict(
-            {"type": "app", "app": "dlg.apps.pyfunc.PyFuncApp", "func_arg_mapping": {}}
+            {
+                "type": "app",
+                "categoryType": CategoryType.APPLICATION,
+                "app": "dlg.apps.pyfunc.PyFuncApp",
+                "appclass": "dlg.apps.pyfunc.PyFuncApp",
+                "func_arg_mapping": {},
+            }
         )
         if self.fname is not None:
             simple_fname = self.fname.split(".")[-1]
             my_dropdict["func_name"] = self.fname
             my_dropdict["nm"] = simple_fname
+            my_dropdict["text"] = simple_fname
         if self.fcode is not None:
             my_dropdict["func_code"] = utils.b2s(base64.b64encode(self.fcode))
         if self.fdefaults:
@@ -309,7 +344,10 @@ class _AppDrop(_DelayedDrop):
             name = self.kwarg_names.pop()
             if name is not None:
                 logger.debug(
-                    "Adding %s/%s to function mapping for %s", name, dep.oid, self.fname
+                    "Adding %s/%s to function mapping for %s",
+                    name,
+                    dep.oid,
+                    self.fname,
                 )
                 self.dropdict["func_arg_mapping"][name] = dep.oid
 
@@ -366,7 +404,13 @@ class _DataDrop(_DelayedDrop):
         logger.debug("Created %r", self)
 
     def make_dropdict(self):
-        my_dropdict = dropdict({"type": "plain", "storage": Categories.MEMORY})
+        my_dropdict = dropdict(
+            {
+                "type": "data",
+                "categoryType": CategoryType.DATA,
+                "storage": Categories.MEMORY,
+            }
+        )
         if not self.producer:
             my_dropdict["pydata"] = pyfunc.serialize_data(self.pydata)
         return my_dropdict
@@ -396,7 +440,10 @@ class _DataDropSequence(_DataDrop):
         return delayed(lambda x, i: x[i])(self, i)
 
     def __repr__(self):
-        return "<_DataDropSequence nout=%d, producer=%r>" % (self.nout, self.producer)
+        return "<_DataDropSequence nout=%d, producer=%r>" % (
+            self.nout,
+            self.producer,
+        )
 
 
 def delayed(x, *args, **kwargs):
