@@ -29,7 +29,6 @@ import socket
 
 from . import deployment_utils
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -46,7 +45,7 @@ class Remote(object):
 
     def _get_ip_from_name(self, hostname):
         rx = re.compile("^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$")
-        if rx.match(hostname):   # already an IP??
+        if rx.match(hostname):  # already an IP??
             return hostname
         else:
             return socket.gethostbyname(hostname)
@@ -58,8 +57,7 @@ class Remote(object):
         if len(set(sorted_peers)) != self.size:
             raise RuntimeError("More than one task started per node, cannot continue")
         # convert nodes to IP addresses if hostnames
-        self.sorted_peers = list(map(lambda x:self._get_ip_from_name(x), 
-            sorted_peers))
+        self.sorted_peers = list(map(lambda x: self._get_ip_from_name(x), sorted_peers))
         nm_range = self._nm_range()
         if nm_range[0] == nm_range[1]:
             raise RuntimeError(
