@@ -31,14 +31,21 @@ import pickle
 import re
 import threading
 import traceback
-from typing import IO, Any, AsyncIterable, BinaryIO, Dict, Iterable, overload
+from typing import Any, Tuple
 import numpy as np
 
 from dlg.ddap_protocol import DROPStates
-from dlg.drop import AppDROP, AbstractDROP, DataDROP, PathBasedDrop
-from dlg.io import IOForURL, OpenMode
+from dlg.data.io import IOForURL, OpenMode
 from dlg import common
 from dlg.common import DropType
+from dlg.apps.app_base import AppDROP
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from dlg.drop import AbstractDROP
+    from dlg.apps.app_base import AppDROP
+    from dlg.data.drops.data_base import DataDROP
 
 logger = logging.getLogger(__name__)
 
@@ -98,7 +105,8 @@ class DROPWaiterCtx(object):
         for drop in self._drops:
             evt = threading.Event()
             drop.subscribe(
-                EvtConsumer(evt, expected_states=self._expected_states), "status"
+                EvtConsumer(evt, expected_states=self._expected_states),
+                "status",
             )
             self._evts.append(evt)
         return self
@@ -117,7 +125,7 @@ class DROPWaiterCtx(object):
                 )
 
 
-def allDropContents(drop, bufsize=4096) -> bytes:
+def allDropContents(drop, bufsize=65536) -> bytes:
     """
     Returns all the data contained in a given DROP
     """
@@ -133,40 +141,52 @@ def allDropContents(drop, bufsize=4096) -> bytes:
     return buf.getvalue()
 
 
-def copyDropContents(source: DataDROP, target: DataDROP, bufsize=4096):
+def copyDropContents(
+    source: "DataDROP", target: "DataDROP", bufsize: int = 65536
+):
     """
     Manually copies data from one DROP into another, in bufsize steps
     """
-    logger.debug(
-        "Copying from %s to %s", repr(source), repr(target))
-    desc = source.open()
-    buf = source.read(desc, bufsize)
-    logger.debug("Read %d bytes from %s", len(buf), 
-    repr(source))
+    logger.debug("Copying from %s to %s", repr(source), repr(target))
+    sdesc = source.open()
+    buf = source.read(sdesc, bufsize)
+    logger.debug("Read %d bytes from %s", len(buf), repr(source))
     st = time.time()
-    tot_w = len(buf)
+    ssize = source.size if source.size is not None else -1
+    logger.debug(
+        "Source size: %d; Source checksum: %d", ssize, source.checksum
+    )
+    tot_w = 0
     ofl = True
+    # target._expectedSize = ssize
     while buf:
-        target.write(buf)
-        tot_w += len(buf)
-        dur = time.time() - st
-        if int(dur) % 5 == 0 and ofl:
-            logger.debug("Wrote %.1f MB to %s; rate %.2f MB/s",
-                tot_w/1024**2, repr(target), tot_w/(1024**2*dur))
+        tot_w += target.write(buf)
+        dur = int(time.time() - st)
+        if dur > 5 and dur % 5 == 0 and ofl:
+            logger.debug(
+                "Wrote %d Bytes to %s; rate %.2f MB/s",
+                tot_w,
+                repr(target),
+                tot_w / (1024**2 * dur),
+            )
             ofl = False
         elif int(dur) % 5 == 4:
             ofl = True
-        buf = source.read(desc, bufsize)
-        # if buf is not None:
-        #     logger.debug(f"Read {len(buf)} bytes from {repr(source)}")
+        buf = source.read(sdesc, bufsize)
     dur = time.time() - st
-    logger.debug("Wrote %.1f MB to %s; rate %.2f MB/s",
-        tot_w/1024**2, repr(target), tot_w/(1024**2*dur))
+    logger.debug(
+        "Wrote %d Bytes of %d to %s; rate %.2f MB/s",
+        tot_w,
+        ssize,
+        repr(target),
+        tot_w / (1024**2 * dur),
+    )
 
-    source.close(desc)
+    source.close(sdesc)
+    return
 
 
-def getUpstreamObjects(drop):
+def getUpstreamObjects(drop: "AbstractDROP"):
     """
     Returns a list of all direct "upstream" DROPs for the given+
     DROP. An DROP A is "upstream" with respect to DROP B if
@@ -178,7 +198,7 @@ def getUpstreamObjects(drop):
     In practice if A is an upstream DROP of B means that it must be moved
     to the COMPLETED state before B can do so.
     """
-    upObjs = []
+    upObjs: list[AbstractDROP] = []
     if isinstance(drop, AppDROP):
         upObjs += drop.inputs
         upObjs += drop.streamingInputs
@@ -220,7 +240,7 @@ def getLeafNodes(drops):
     ]
 
 
-def depthFirstTraverse(node: AbstractDROP, visited=[]):
+def depthFirstTraverse(node: "AbstractDROP", visited=[]):
     """
     Depth-first iterator for a DROP graph.
 
@@ -283,12 +303,12 @@ def listify(o):
     return [o]
 
 
-def save_pickle(drop: DataDROP, data: Any):
+def save_pickle(drop: "DataDROP", data: Any):
     """Saves a python object in pkl format"""
     pickle.dump(data, drop)
 
 
-def load_pickle(drop: DataDROP) -> Any:
+def load_pickle(drop: "DataDROP") -> Any:
     """Loads a pkl formatted data object stored in a DataDROP.
     Note: does not support streaming mode.
     """
@@ -314,7 +334,7 @@ def load_pickle(drop: DataDROP) -> Any:
 #             yield pickle.load(p)
 
 
-def save_npy(drop: DataDROP, ndarray: np.ndarray, allow_pickle=False):
+def save_npy(drop: "DataDROP", ndarray: np.ndarray, allow_pickle=False):
     """
     Saves a numpy ndarray to a drop in npy format
     """
@@ -328,11 +348,11 @@ def save_npy(drop: DataDROP, ndarray: np.ndarray, allow_pickle=False):
     dropio.close()
 
 
-def save_numpy(drop: DataDROP, ndarray: np.ndarray):
+def save_numpy(drop: "DataDROP", ndarray: np.ndarray):
     save_npy(drop, ndarray)
 
 
-def load_npy(drop: DataDROP, allow_pickle=False) -> np.ndarray:
+def load_npy(drop: "DataDROP", allow_pickle=False) -> np.ndarray:
     """
     Loads a numpy ndarray from a drop in npy format
     """
@@ -343,7 +363,7 @@ def load_npy(drop: DataDROP, allow_pickle=False) -> np.ndarray:
     return res
 
 
-def load_numpy(drop: DataDROP):
+def load_numpy(drop: "DataDROP"):
     return load_npy(drop)
 
 
@@ -422,7 +442,7 @@ class DROPFile(object):
             self._drop.close(self._fd)
         self._isClosed = True
 
-    def read(self, size=4096):
+    def read(self, size=65536):
         if self._io:
             return self._io.read(size)
         return self._drop.read(self._fd, size)
@@ -529,75 +549,6 @@ def replace_dataurl_placeholders(cmd, inputs, outputs):
 
     return cmd
 
-def serialize_kwargs(keyargs, prefix="--", separator=" "):
-    kwargs = []
-    for (name, value) in iter(keyargs.items()):
-        if prefix == "--" and len(name) == 1:
-            kwargs += [f"-{name} {value}"]
-        else:
-            kwargs += [f"{prefix}{name}{separator}{value}".strip()]
-    return kwargs
-
-
-def serialize_applicationArgs(applicationArgs, prefix="--", separator=" "):
-    """
-    Unpacks the applicationArgs dictionary and returns a string
-    that can be used as command line parameters.
-    """
-    if not isinstance(applicationArgs, dict):
-        logger.info("applicationArgs are not passed as a dict. Ignored!")
-    else:
-        logger.info("ApplicationArgs found %s", applicationArgs)
-    # construct the actual command line from all application parameters
-    kwargs = {}
-    pargs = []
-    positional = False
-    precious = False
-    for (name, vdict) in applicationArgs.items():
-        if vdict in [None, False, ""]:
-            continue
-        elif isinstance(vdict, bool):
-            value = ""
-        elif isinstance(vdict, dict):
-            precious = vdict["precious"]
-            value = vdict["value"]
-            if value in [None, False, ""] and not precious:
-                continue
-            positional = vdict["positional"]
-        if positional:
-            pargs.append(str(value).strip())
-        else:
-            kwargs.update({name:value})
-    kwargs = serialize_kwargs(kwargs, prefix=prefix, separator=separator)
-    logger.info('Constructed command line arguments: %s %s', pargs, kwargs)
-    # return f"{' '.join(pargs + kwargs)}"  # add kwargs to end of pargs
-    return (pargs, kwargs)
-
-def identify_named_ports(ports, port_dict, posargs, pargsDict, appArgs, check_len=0, mode="inputs"):
-    """
-    """
-    logger.debug("Using named ports to remove %s from arguments: %s %d", mode, 
-        port_dict, check_len)
-    # pargsDict = collections.OrderedDict(zip(posargs,[None]*len(posargs)))
-    kwargs = {}
-    for i in range(check_len):
-        # key for final dict is value in named ports dict
-        key = list(port_dict[i].values())[0]
-        # value for final dict is value in inputs dict
-        value = ports[list(port_dict[i].keys())[0]]
-        if not value: value = '' # make sure we are passing NULL drop events
-        if key in posargs:
-            pargsDict.update({key:value})
-            logger.debug("Using %s '%s' for parg %s", mode, value, key)
-            posargs.pop(posargs.index(key))
-        else:
-            kwargs.update({key:value})
-            logger.debug("Using %s '%s' for kwarg %s", mode, value, key)
-        _dum = appArgs.pop(key) if key in appArgs else None
-        logger.debug("Argument used as %s removed: %s", mode, _dum)
-    logger.debug("Returning mapped ports: %s", kwargs)
-    return kwargs
-
 
 # Easing the transition from single- to multi-package
 get_leaves = common.get_leaves
