@@ -41,7 +41,7 @@ section 8 "Data Life Cycle Management", where the DLM is defined:
       SDP_REQ-252, and the need for data tracing, provenance and access control,
       SKA1-SYS_REQ-2821, SDP_REQ-255
    3. Migration between storage layers, includes SDP_REQ-263
-   4. Replication/distribution including resilience (preciousness) support,
+   4. Replication/distribution including resilience (persistence) support,
       incl. SKA1-SYS_REQ-2350, SDP_REQ-260 - 262
    5. Retirement of expired (temporary) data, incl. SDP_REQ-256."
 
@@ -126,9 +126,11 @@ import time
 
 from . import registry
 from .hsm import manager
+from .hsm.store import AbstractStore
 from .. import droputils
 from ..ddap_protocol import DROPStates, DROPPhases, AppDROPStates
-from ..drop import ContainerDROP
+from ..drop import AbstractDROP
+from ..data.drops.container import ContainerDROP
 
 logger = logging.getLogger(__name__)
 
@@ -209,7 +211,9 @@ class DataLifecycleManager:
     An object that deals with automatic data drop replication and deletion.
     """
 
-    def __init__(self, check_period=0, cleanup_period=0, enable_drop_replication=False):
+    def __init__(
+        self, check_period=0, cleanup_period=0, enable_drop_replication=False
+    ):
         self._reg = registry.InMemoryRegistry()
         self._listener = DropEventListener(self)
         self._enable_drop_replication = enable_drop_replication
@@ -222,7 +226,7 @@ class DataLifecycleManager:
         # instead of _drops.itervalues() to get a full, thread-safe copy of the
         # dictionary values. Maybe there's a better approach for thread-safety
         # here
-        self._drops = {}
+        self._drops: dict[str, AbstractDROP] = {}
 
         self._check_period = check_period
         self._cleanup_period = cleanup_period
@@ -234,16 +238,12 @@ class DataLifecycleManager:
         # Spawn the background threads
         if self._check_period:
             self._drop_checker = DROPChecker(
-                "DropChecker",
-                self,
-                self._check_period
+                "DropChecker", self, self._check_period
             )
             self._drop_checker.start()
         if self._cleanup_period:
             self._drop_garbage_collector = DROPGarbageCollector(
-                "DropGarbageCollector",
-                self,
-                self._cleanup_period
+                "DropGarbageCollector", self, self._cleanup_period
             )
             self._drop_garbage_collector.start()
 
@@ -290,9 +290,10 @@ class DataLifecycleManager:
 
             # Expire-after-use: mark as expired if all consumers
             # are finished using this DROP
-            if drop.expireAfterUse:
+            if not drop.persist and drop.expireAfterUse:
                 allDone = all(
-                    c.execStatus in [AppDROPStates.FINISHED, AppDROPStates.ERROR]
+                    c.execStatus
+                    in [AppDROPStates.FINISHED, AppDROPStates.ERROR]
                     for c in drop.consumers
                 )
                 if not allDone:
@@ -348,17 +349,21 @@ class DataLifecycleManager:
                         replicas.append(siblingDrop)
                     else:
                         logger.warning(
-                            "%r (replicated from %r) has disappeared", siblingDrop, drop
+                            "%r (replicated from %r) has disappeared",
+                            siblingDrop,
+                            drop,
                         )
                         toRemove.append(siblingDrop.uid)
 
                 if len(replicas) > 1:
                     logger.info(
-                        "%r has still more than one replica, no action needed", drop
+                        "%r has still more than one replica, no action needed",
+                        drop,
                     )
                 elif len(replicas) == 1:
                     logger.info(
-                        "Only one replica left for DROP %r, will create a new one", drop
+                        "Only one replica left for DROP %r, will create a new one",
+                        drop,
                     )
                     self.replicateDrop(replicas[0])
                 else:
@@ -482,8 +487,10 @@ class DataLifecycleManager:
             return
 
         drop = self._drops[uid]
-        if drop.precious and self.isReplicable(drop):
-            logger.debug("Replicating %r because it's precious", drop)
+        if drop.persist and self.isReplicable(drop):
+            logger.debug(
+                "Replicating %r because it's marked to be persisted", drop
+            )
             try:
                 self.replicateDrop(drop)
             except:
@@ -512,7 +519,9 @@ class DataLifecycleManager:
         availableSpace = store.getAvailableSpace()
 
         if size > availableSpace:
-            raise Exception("Cannot replicate DROP to store %s: not enough space left")
+            raise Exception(
+                "Cannot replicate DROP to store %s: not enough space left"
+            )
 
         # Create new DROP and write the contents of the original into it
         # TODO: In a real world application this will probably happen in a separate
@@ -532,12 +541,14 @@ class DataLifecycleManager:
     def getDropUids(self, drop):
         return self._reg.getDropUids(drop)
 
-    def _replicate(self, drop, store):
+    def _replicate(self, drop: AbstractDROP, store: AbstractStore):
 
         # Dummy, but safe, new UID
         newUid = "uid:" + "".join(
             [
-                random.SystemRandom().choice(string.ascii_letters + string.digits)
+                random.SystemRandom().choice(
+                    string.ascii_letters + string.digits
+                )
                 for _ in range(10)
             ]
         )
@@ -546,7 +557,7 @@ class DataLifecycleManager:
 
         # For the time being we manually copy the contents of the current DROP into it
         newDrop = store.createDrop(
-            drop.oid, newUid, expectedSize=drop.size, precious=drop.precious
+            drop.oid, newUid, expectedSize=drop.size, persist=drop.persist
         )
         droputils.copyDropContents(drop, newDrop)
 
