@@ -30,6 +30,7 @@ import io
 import json
 import logging
 import os
+import re
 import tarfile
 import threading
 
@@ -54,6 +55,7 @@ from ..exceptions import (
 from ..restserver import RestServer
 from ..restutils import RestClient, RestClientException
 from .session import generateLogFileName
+from dlg.common.deployment_methods import DeploymentMethods
 
 logger = logging.getLogger(__name__)
 
@@ -75,9 +77,14 @@ def daliuge_aware(func):
             if res is not None:
                 bottle.response.content_type = "application/json"
                 # set CORS headers
+                origin = bottle.request.headers.raw("Origin")
+                if origin is None:
+                    origin = "http://localhost:8084"
+                elif not re.match(r"http://((localhost)|(127.0.0.1)):80[0-9][0-9]", origin):
+                    origin = "http://localhost:8084"
                 bottle.response.headers[
                     "Access-Control-Allow-Origin"
-                ] = "http://localhost:8084"
+                ] = origin
                 bottle.response.headers["Access-Control-Allow-Credentials"] = "true"
                 bottle.response.headers[
                     "Access-Control-Allow-Methods"
@@ -145,6 +152,7 @@ class ManagerRestServer(RestServer):
 
         # Mappings
         app = self.app
+        app.get("/api/submission_method", callback=self.submit_methods)
         app.post("/api/stop", callback=self.stop_manager)
         app.post("/api/sessions", callback=self.createSession)
         app.get("/api/sessions", callback=self.getSessions)
@@ -187,6 +195,10 @@ class ManagerRestServer(RestServer):
         The default implementation does nothing.
         """
 
+    @daliuge_aware
+    def submit_methods(self):
+        return {"methods": [DeploymentMethods.BROWSER]}
+
     def _stop_manager(self):
         self.dm.shutdown()
         self.stop()
@@ -434,24 +446,24 @@ class CompositeManagerRestServer(ManagerRestServer):
     def initializeSpecifics(self, app):
         app.get("/api", callback=self.getCMStatus)
         app.get("/api/nodes", callback=self.getCMNodes)
-        app.post("/api/nodes/<node>", callback=self.addCMNode)
-        app.delete("/api/nodes/<node>", callback=self.removeCMNode)
+        app.post("/api/node/<node>", callback=self.addCMNode)
+        app.delete("/api/node/<node>", callback=self.removeCMNode)
 
         # Query forwarding to sub-nodes
-        app.get("/api/nodes/<node>/sessions", callback=self.getNodeSessions)
+        app.get("/api/node/<node>/sessions", callback=self.getNodeSessions)
         app.get(
-            "/api/nodes/<node>/sessions/<sessionId>",
+            "/api/node/<node>/sessions/<sessionId>",
             callback=self.getNodeSessionInformation,
         )
         app.get(
-            "/api/nodes/<node>/sessions/<sessionId>/status",
+            "/api/node/<node>/sessions/<sessionId>/status",
             callback=self.getNodeSessionStatus,
         )
         app.get(
-            "/api/nodes/<node>/sessions/<sessionId>/graph", callback=self.getNodeGraph
+            "/api/node/<node>/sessions/<sessionId>/graph", callback=self.getNodeGraph
         )
         app.get(
-            "/api/nodes/<node>/sessions/<sessionId>/graph/status",
+            "/api/node/<node>/sessions/<sessionId>/graph/status",
             callback=self.getNodeGraphStatus,
         )
 
@@ -577,19 +589,20 @@ class MasterManagerRestServer(CompositeManagerRestServer):
     def initializeSpecifics(self, app):
         CompositeManagerRestServer.initializeSpecifics(self, app)
         # DIM manamagement
-        app.post("/api/islands/<dim>", callback=self.addDIM)
-        app.delete("/api/islands/<dim>", callback=self.removeDIM)
+        app.post("/api/island/<dim>", callback=self.addDIM)
+        app.delete("/api/island/<dim>", callback=self.removeDIM)
         # Query forwarding to daemons
-        app.post("/api/managers/<host>/dataisland", callback=self.createDataIsland)
+        app.post("/api/managers/<host>/island", callback=self.createDataIsland)
         app.post("/api/managers/<host>/node/start", callback=self.startNM)
         app.post("/api/managers/<host>/node/stop", callback=self.stopNM)
-        app.post("/api/managers/<host>/nodes/<node>", callback=self.addNM)
-        app.delete("/api/managers/<host>/nodes/<node>", callback=self.removeNM)
+        # manage node manager assignment
+        app.post("/api/managers/<host>/node/<node>", callback=self.addNM)
+        app.delete("/api/managers/<host>/node/<node>", callback=self.removeNM)
         # Querying about managers
         app.get("/api/islands", callback=self.getDIMs)
         app.get("/api/nodes", callback=self.getNMs)
         app.get("/api/managers/<host>/node", callback=self.getNMInfo)
-        app.get("/api/managers/<host>/dataisland", callback=self.getDIMInfo)
+        app.get("/api/managers/<host>/island", callback=self.getDIMInfo)
         app.get("/api/managers/<host>/master", callback=self.getMMInfo)
 
     @daliuge_aware
@@ -597,8 +610,9 @@ class MasterManagerRestServer(CompositeManagerRestServer):
         with RestClient(
             host=host, port=constants.DAEMON_DEFAULT_REST_PORT, timeout=10
         ) as c:
-            c._post_json("/managers/dataisland", bottle.request.body.read())
+            c._post_json("/managers/island/start", bottle.request.body.read())
         self.dm.addDmHost(host)
+        return {"islands": self.dm.dmHosts}
 
     @daliuge_aware
     def getDIMs(self):
@@ -639,7 +653,7 @@ class MasterManagerRestServer(CompositeManagerRestServer):
         with RestClient(host=host, port=port, timeout=10, url_prefix="/api") as c:
             return json.loads(
                 c._POST(
-                    f"/nodes/{node}",
+                    f"/node/{node}",
                 ).read()
             )
 
@@ -648,7 +662,7 @@ class MasterManagerRestServer(CompositeManagerRestServer):
         port = constants.ISLAND_DEFAULT_REST_PORT
         logger.debug("Removing NM %s from DIM %s", node, host)
         with RestClient(host=host, port=port, timeout=10, url_prefix="/api") as c:
-            return json.loads(c._DELETE(f"/nodes/{node}").read())
+            return json.loads(c._DELETE(f"/node/{node}").read())
 
     @daliuge_aware
     def getNMInfo(self, host):
@@ -662,7 +676,7 @@ class MasterManagerRestServer(CompositeManagerRestServer):
         with RestClient(
             host=host, port=constants.DAEMON_DEFAULT_REST_PORT, timeout=10
         ) as c:
-            return json.loads(c._GET("/managers/dataisland").read())
+            return json.loads(c._GET("/managers/island").read())
 
     @daliuge_aware
     def getMMInfo(self, host):
