@@ -44,12 +44,12 @@ from ..common.reproducibility.constants import ReproducibilityFlags, ALL_RMODES
 from ..ddap_protocol import DROPLinkType, DROPRel, DROPStates
 from ..drop import (
     AbstractDROP,
-    AppDROP,
-    EndDROP,
-    InputFiredAppDROP,
     LINKTYPE_1TON_APPEND_METHOD,
     LINKTYPE_1TON_BACK_APPEND_METHOD,
 )
+from ..apps.app_base import AppDROP, InputFiredAppDROP
+from ..data.drops.data_base import EndDROP
+
 from ..exceptions import (
     InvalidSessionState,
     InvalidGraphException,
@@ -98,13 +98,21 @@ class ReproFinishedListener(object):
         self._completed += 1
         self._session.append_reprodata(evt.oid, evt.reprodata)
         logger.debug(
-            "%d/%d drops filed reproducibility", self._completed, self._nexpected
+            "%d/%d drops filed reproducibility",
+            self._completed,
+            self._nexpected,
         )
         if self._completed == self._nexpected:
             if not self._session.reprostatus:
                 logger.debug("Building Reproducibility BlockDAG")
-                new_reprodata = init_runtime_repro_data(self._session._graph, self._session._graphreprodata).get("reprodata", {})
-                logger.debug("Reprodata for %s is %s", self._session.sessionId, json.dumps(new_reprodata))
+                new_reprodata = init_runtime_repro_data(
+                    self._session._graph, self._session._graphreprodata
+                ).get("reprodata", {})
+                logger.debug(
+                    "Reprodata for %s is %s",
+                    self._session.sessionId,
+                    json.dumps(new_reprodata),
+                )
                 self._session._graphreprodata = new_reprodata
                 self._session.reprostatus = True
                 self._session.write_reprodata()
@@ -164,6 +172,15 @@ class Session(object):
         self._graphreprodata = None
         self._reprofinished = False
 
+        # create the session directory and change CWD
+        self._sessionDir = f"{utils.getDlgWorkDir()}/{sessionId}"
+        logger.debug("Creating session directory: %s", self._sessionDir)
+        createDirIfMissing(self._sessionDir)
+        os.chdir(self._sessionDir)
+
+        logger.debug("Updating ENV with SESSION_ID: %s", sessionId)
+        os.environ.update({"DLG_SESSION_ID": sessionId})
+
         class SessionFilter(logging.Filter):
             def __init__(self, sessionId):
                 self.sessionId = sessionId
@@ -177,10 +194,10 @@ class Session(object):
         fmt = logging.Formatter(fmt)
         fmt.converter = time.gmtime
 
-        logdir = utils.getDlgLogsDir()
-        if self._nm is not None:
-            logdir = self._nm.logdir
-        logfile = generateLogFileName(logdir, self.sessionId)
+        # logdir = utils.getDlgLogsDir()
+        # if self._nm is not None:
+        #     logdir = self._nm.logdir
+        logfile = generateLogFileName(self._sessionDir, self.sessionId)
         try:
             self.file_handler = logging.FileHandler(logfile)
             self.file_handler.setFormatter(fmt)
@@ -195,6 +212,10 @@ class Session(object):
     def sessionId(self):
         return self._sessionId
 
+    @property
+    def sessionDir(self):
+        return self._sessionDir
+
     @property
     def status(self):
         with self._statusLock:
@@ -227,8 +248,7 @@ class Session(object):
             self._reprofinished = status
 
     def write_reprodata(self):
-        parts = [utils.getDlgLogsDir(), self._sessionId]
-        the_dir = os.path.abspath(os.path.normpath(os.path.join(*parts)))
+        the_dir = self._sessionDir
         createDirIfMissing(the_dir)
         the_path = os.path.join(the_dir, "reprodata.out")
         with open(the_path, "w+", encoding="utf-8") as file:
@@ -267,17 +287,22 @@ class Session(object):
         self.status = SessionStates.BUILDING
 
         # This will check the consistency of each dropSpec
-        graphSpecDict, self._graphreprodata = graph_loader.loadDropSpecs(graphSpec)
+        graphSpecDict, self._graphreprodata = graph_loader.loadDropSpecs(
+            graphSpec
+        )
         # Check for duplicates
         duplicates = set(graphSpecDict) & set(self._graph)
         if duplicates:
             raise InvalidGraphException(
-                "Trying to add drops with OIDs that already exist: %r" % (duplicates,)
+                "Trying to add drops with OIDs that already exist: %r"
+                % (duplicates,)
             )
 
         self._graph.update(graphSpecDict)
 
-        logger.debug("Added a graph definition with %d DROPs", len(graphSpecDict))
+        logger.debug(
+            "Added a graph definition with %d DROPs", len(graphSpecDict)
+        )
 
     @track_current_session
     def linkGraphParts(self, lhOID, rhOID, linkType, force=False):
@@ -303,7 +328,9 @@ class Session(object):
             missingOids.append(rhOID)
         if missingOids:
             oids = "OID" if len(missingOids) == 1 else "OIDs"
-            raise InvalidGraphException("No DROP found for %s %r" % (oids, missingOids))
+            raise InvalidGraphException(
+                "No DROP found for %s %r" % (oids, missingOids)
+            )
 
         graph_loader.addLink(linkType, lhDropSpec, rhOID, force=force)
 
@@ -325,10 +352,11 @@ class Session(object):
         # in reality this particular session is managing nothing
         status = self.status
         if (self._graph and status != SessionStates.BUILDING) or (
-                not self._graph and status != SessionStates.PRISTINE
+            not self._graph and status != SessionStates.PRISTINE
         ):
             raise InvalidSessionState(
-                "Can't deploy this session in its current status: %d" % (status)
+                "Can't deploy this session in its current status: %d"
+                % (status)
             )
 
         if not self._graph and completedDrops:
@@ -403,10 +431,14 @@ class Session(object):
 
         # Append proxies
         logger.info(
-            "Creating %d drop proxies: %r", len(self._proxyinfo), self._proxyinfo
+            "Creating %d drop proxies: %r",
+            len(self._proxyinfo),
+            self._proxyinfo,
         )
         for host, port, local_uid, relname, remote_uid in self._proxyinfo:
-            proxy = rpc.DropProxy(self._nm, host, port, self._sessionId, remote_uid)
+            proxy = rpc.DropProxy(
+                self._nm, host, port, self._sessionId, remote_uid
+            )
             logger.debug(
                 "Attaching proxy to local %r via %s(proxy, False)",
                 self._drops[local_uid],
@@ -434,7 +466,9 @@ class Session(object):
     def trigger_drops(self, uids):
         for drop, downStreamDrops in droputils.breadFirstTraverse(self._roots):
             downStreamDrops[:] = [
-                dsDrop for dsDrop in downStreamDrops if isinstance(dsDrop, AbstractDROP)
+                dsDrop
+                for dsDrop in downStreamDrops
+                if isinstance(dsDrop, AbstractDROP)
             ]
             if drop.uid in uids:
                 if isinstance(drop, InputFiredAppDROP):
@@ -495,7 +529,7 @@ class Session(object):
 
                 # We are in the event receiver side
                 if (rel.rel in evt_consumer and rel.lhs is local_uid) or (
-                        rel.rel in evt_producer and rel.rhs is local_uid
+                    rel.rel in evt_producer and rel.rhs is local_uid
                 ):
                     dropsubs[remote_uid].add(local_uid)
 
@@ -511,14 +545,16 @@ class Session(object):
                     remote_uid = rel.rhs
                     mname = LINKTYPE_1TON_BACK_APPEND_METHOD[rel.rel]
 
-                self._proxyinfo.append((host, rpc_port, local_uid, mname, remote_uid))
+                self._proxyinfo.append(
+                    (host, rpc_port, local_uid, mname, remote_uid)
+                )
 
     def append_reprodata(self, oid, reprodata):
         if oid in self._graph:
             if self._graph[oid].get("reprodata") is None:
                 return
             if self._graph[oid]["reprodata"]["rmode"] == str(
-                    ReproducibilityFlags.ALL.value
+                ReproducibilityFlags.ALL.value
             ):
                 drop_reprodata = reprodata.get("data", {})
                 drop_hashes = reprodata.get("merkleroot", {})
@@ -531,10 +567,12 @@ class Session(object):
                     ] = drop_hashes.get(rmode.name, b"")
 
             else:
-                self._graph[oid]["reprodata"]["rg_data"] = reprodata.get("data", {})
-                self._graph[oid]["reprodata"]["rg_data"]["merkleroot"] = reprodata.get(
-                    "merkleroot", b""
+                self._graph[oid]["reprodata"]["rg_data"] = reprodata.get(
+                    "data", {}
                 )
+                self._graph[oid]["reprodata"]["rg_data"][
+                    "merkleroot"
+                ] = reprodata.get("merkleroot", b"")
 
     @track_current_session
     def finish(self):
@@ -542,7 +580,9 @@ class Session(object):
         logger.info("Session %s finished", self._sessionId)
         for drop, downStreamDrops in droputils.breadFirstTraverse(self._roots):
             downStreamDrops[:] = [
-                dsDrop for dsDrop in downStreamDrops if isinstance(dsDrop, AbstractDROP)
+                dsDrop
+                for dsDrop in downStreamDrops
+                if isinstance(dsDrop, AbstractDROP)
             ]
             if drop.status in (DROPStates.INITIALIZED, DROPStates.WRITING):
                 drop.setCompleted()
@@ -553,16 +593,18 @@ class Session(object):
         logger.info("Session %s ended", self._sessionId)
         for drop, downStreamDrops in droputils.breadFirstTraverse(self._roots):
             downStreamDrops[:] = [
-                dsDrop for dsDrop in downStreamDrops if isinstance(dsDrop, AbstractDROP)
+                dsDrop
+                for dsDrop in downStreamDrops
+                if isinstance(dsDrop, AbstractDROP)
             ]
             if drop.status in (DROPStates.INITIALIZED, DROPStates.WRITING):
                 drop.skip()
 
     def getGraphStatus(self):
         if self.status not in (
-                SessionStates.RUNNING,
-                SessionStates.FINISHED,
-                SessionStates.CANCELLED,
+            SessionStates.RUNNING,
+            SessionStates.FINISHED,
+            SessionStates.CANCELLED,
         ):
             raise InvalidSessionState(
                 "The session is currently not running, cannot get graph status"
@@ -577,7 +619,9 @@ class Session(object):
         statusDict = collections.defaultdict(dict)
         for drop, downStreamDrops in droputils.breadFirstTraverse(self._roots):
             downStreamDrops[:] = [
-                dsDrop for dsDrop in downStreamDrops if isinstance(dsDrop, AbstractDROP)
+                dsDrop
+                for dsDrop in downStreamDrops
+                if isinstance(dsDrop, AbstractDROP)
             ]
             if isinstance(drop, AppDROP):
                 statusDict[drop.oid]["execStatus"] = drop.execStatus
@@ -590,16 +634,19 @@ class Session(object):
         status = self.status
         if status != SessionStates.RUNNING:
             raise InvalidSessionState(
-                "Can't cancel this session in its current status: %d" % (status)
+                "Can't cancel this session in its current status: %d"
+                % (status)
             )
         for drop, downStreamDrops in droputils.breadFirstTraverse(self._roots):
             downStreamDrops[:] = [
-                dsDrop for dsDrop in downStreamDrops if isinstance(dsDrop, AbstractDROP)
+                dsDrop
+                for dsDrop in downStreamDrops
+                if isinstance(dsDrop, AbstractDROP)
             ]
             if drop.status not in (
-                    DROPStates.ERROR,
-                    DROPStates.COMPLETED,
-                    DROPStates.CANCELLED,
+                DROPStates.ERROR,
+                DROPStates.COMPLETED,
+                DROPStates.CANCELLED,
             ):
                 drop.cancel()
         self.status = SessionStates.CANCELLED
@@ -633,7 +680,9 @@ class Session(object):
             drop = self._drops[uid]
             return getattr(drop, prop_name)
         except AttributeError:
-            raise DaliugeException("%r has no property called %s" % (drop, prop_name))
+            raise DaliugeException(
+                "%r has no property called %s" % (drop, prop_name)
+            )
 
     def call_drop(self, uid, method, *args):
         if uid not in self._drops:
@@ -642,7 +691,9 @@ class Session(object):
             drop = self._drops[uid]
             m = getattr(drop, method)
         except AttributeError:
-            raise DaliugeException("%r has no method called %s" % (drop, method))
+            raise DaliugeException(
+                "%r has no method called %s" % (drop, method)
+            )
         return m(*args)
 
     # Support for the 'with' keyword
