@@ -54,23 +54,23 @@ class ManagerStarter(object):
     ):
         manager = manager_class(*manager_args, **manager_kwargs)
         server = rest_class(manager)
-        thread = threading.Thread(target=server.start, args=("127.0.0.1", port))
+        thread = threading.Thread(target=server.start, args=("localhost", port))
         thread.start()
-        self.assertTrue(portIsOpen("127.0.0.1", port, 5))
+        self.assertTrue(portIsOpen("localhost", port, 5))
         return ManagerInfo(manager, server, thread, self)
 
     def start_nm_in_thread(self, port=constants.NODE_DEFAULT_REST_PORT):
         return self._start_manager_in_thread(port, NodeManager, NMRestServer, False)
 
     def start_dim_in_thread(
-        self, nm_hosts=["127.0.0.1"], port=constants.ISLAND_DEFAULT_REST_PORT
+        self, nm_hosts=["localhost"], port=constants.ISLAND_DEFAULT_REST_PORT
     ):
         return self._start_manager_in_thread(
             port, DataIslandManager, CompositeManagerRestServer, nm_hosts
         )
 
     def start_mm_in_thread(
-        self, nm_hosts=["127.0.0.1"], port=constants.MASTER_DEFAULT_REST_PORT
+        self, nm_hosts=["localhost"], port=constants.MASTER_DEFAULT_REST_PORT
     ):
         return self._start_manager_in_thread(
             port, MasterManager, CompositeManagerRestServer, nm_hosts
