@@ -24,15 +24,12 @@ import json
 import pkg_resources
 
 from dlg import graph_loader
-from dlg.common import Categories
 from dlg.ddap_protocol import DROPLinkType, DROPRel
-from dlg.drop import (
-    InMemoryDROP,
-    SharedMemoryDROP,
-    ContainerDROP,
-    AppDROP,
-    DirectoryContainer,
-)
+from dlg.data.drops.container import ContainerDROP
+from dlg.apps.app_base import AppDROP
+
+from dlg.data.drops.memory import InMemoryDROP, SharedMemoryDROP
+from dlg.data.drops.directorycontainer import DirectoryContainer
 from dlg.common import Categories
 from dlg.apps.simple import RandomArrayApp
 
@@ -44,14 +41,18 @@ class DummyApp(AppDROP):
 
 class TestGraphLoader(unittest.TestCase):
     def test_singleMemoryDrop(self):
-        dropSpecList = [{"oid": "A", "type": "plain", "storage": Categories.MEMORY}]
+        dropSpecList = [
+            {"oid": "A", "type": "data", "storage": Categories.MEMORY}
+        ]
         a = graph_loader.createGraphFromDropSpecList(dropSpecList)[0]
         self.assertIsInstance(a, InMemoryDROP)
         self.assertEqual("A", a.oid)
         self.assertEqual("A", a.uid)
 
     def test_sharedMemoryDrop(self):
-        dropSpecList = [{"oid": "A", "type": "plain", "storage": Categories.SHMEM}]
+        dropSpecList = [
+            {"oid": "A", "type": "data", "storage": Categories.SHMEM}
+        ]
         a = graph_loader.createGraphFromDropSpecList(dropSpecList)[0]
         self.assertIsInstance(a, SharedMemoryDROP)
         self.assertEqual("A", a.oid)
@@ -59,7 +60,7 @@ class TestGraphLoader(unittest.TestCase):
 
     def test_containerDrop(self):
         dropSpecList = [
-            {"oid": "A", "type": "plain", "storage": Categories.MEMORY},
+            {"oid": "A", "type": "data", "storage": Categories.MEMORY},
             {"oid": "B", "type": "container", "children": ["A"]},
         ]
         a = graph_loader.createGraphFromDropSpecList(dropSpecList)[0]
@@ -74,11 +75,16 @@ class TestGraphLoader(unittest.TestCase):
 
         # A directory container
         dropSpecList = [
-            {"oid": "A", "type": "plain", "storage": Categories.FILE, "dirname": "."},
+            {
+                "oid": "A",
+                "type": "data",
+                "storage": Categories.FILE,
+                "dirname": ".",
+            },
             {
                 "oid": "B",
                 "type": "container",
-                "container": "dlg.drop.DirectoryContainer",
+                "container": "dlg.data.drops.DirectoryContainer",
                 "children": ["A"],
                 "dirname": ".",
             },
@@ -91,11 +97,15 @@ class TestGraphLoader(unittest.TestCase):
         dropSpecList = [
             {
                 "oid": "A",
-                "type": "plain",
+                "type": "data",
                 "storage": Categories.MEMORY,
                 "consumers": ["B"],
             },
-            {"oid": "B", "type": "app", "app": "test.test_graph_loader.DummyApp"},
+            {
+                "oid": "B",
+                "type": "app",
+                "app": "test.test_graph_loader.DummyApp",
+            },
         ]
         a = graph_loader.createGraphFromDropSpecList(dropSpecList)[0]
         self.assertIsInstance(a, InMemoryDROP)
@@ -122,12 +132,19 @@ class TestGraphLoader(unittest.TestCase):
 
         unmetRelationships = graph_loader.removeUnmetRelationships(graphDesc)
         self.assertEqual(4, len(unmetRelationships))
-        self.assertIn(DROPRel("D", DROPLinkType.CONSUMER, "A"), unmetRelationships)
         self.assertIn(
-            DROPRel("D", DROPLinkType.STREAMING_CONSUMER, "C"), unmetRelationships
+            DROPRel("D", DROPLinkType.CONSUMER, "A"), unmetRelationships
+        )
+        self.assertIn(
+            DROPRel("D", DROPLinkType.STREAMING_CONSUMER, "C"),
+            unmetRelationships,
+        )
+        self.assertIn(
+            DROPRel("Z", DROPLinkType.PRODUCER, "A"), unmetRelationships
+        )
+        self.assertIn(
+            DROPRel("X", DROPLinkType.PRODUCER, "A"), unmetRelationships
         )
-        self.assertIn(DROPRel("Z", DROPLinkType.PRODUCER, "A"), unmetRelationships)
-        self.assertIn(DROPRel("X", DROPLinkType.PRODUCER, "A"), unmetRelationships)
 
         # The original dropSpecs have changed as well
         a = graphDesc[0]
@@ -136,7 +153,9 @@ class TestGraphLoader(unittest.TestCase):
         self.assertEqual(1, len(a["consumers"]))
         self.assertEqual("B", a["consumers"][0])
         self.assertFalse("producers" in a and len(a["producers"]) > 0)
-        self.assertFalse("streamingConsumers" in c and len(c["streamingConsumers"]) > 0)
+        self.assertFalse(
+            "streamingConsumers" in c and len(c["streamingConsumers"]) > 0
+        )
 
     def test_removeUnmetRelationships_named(self):
 
@@ -188,3 +207,30 @@ class TestGraphLoader(unittest.TestCase):
         self.assertEqual(app.integer, True)
         self.assertEqual(app.low, 34)
         self.assertEqual(app.high, 3456)
+
+    def test_backwardsCompatibilityWithPreciousFlag(self):
+        """
+        Ensure that precious is detected and exposed as the persist flag
+        """
+        testCases = [
+            ("precious", True),
+            ("precious", False),
+            ("persist", True),
+            ("persist", False),
+            (None, False),  # Default of False is specific to MemoryDrops
+        ]
+        for key, value in testCases:
+            with self.subTest(key=key, value=value):
+                dropSpec = {
+                    "oid": "A",
+                    "type": "data",
+                    "storage": Categories.MEMORY,
+                }
+                if key is not None:
+                    dropSpec[key] = value
+
+                graph = graph_loader.createGraphFromDropSpecList([dropSpec])
+                data = graph[0]
+                self.assertIsInstance(data, InMemoryDROP)
+                self.assertEqual(value, data.persist)
+                self.assertFalse(hasattr(data, "precious"))
