@@ -162,7 +162,7 @@ class TestUtils(unittest.TestCase):
                     self.assertEqual(0, len(uncompressed_stream.read(100)))
 
     def test_json_stream_simple_sequence(self):
-        for s in ([0], [{}], ["a"], [{"oid": "A", "type": "plain"}]):
+        for s in ([0], [{}], ["a"], [{"oid": "A", "type": "data"}]):
             stream = utils.JSONStream(s)
             self.assertEqual(s, json.loads(stream.read(100).decode("utf8")))
 
